#!/usr/bin/python3
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-

import distro_info
import glob
import logging
import multiprocessing
import os
import random
import sys
import tempfile
import time
import unittest

try:
    from test.support import EnvironmentVarGuard
except ImportError:
    from test.support.os_helper import EnvironmentVarGuard
from urllib.error import HTTPError
from urllib.request import install_opener, urlopen
from mock import patch, mock_open
from http.server import BaseHTTPRequestHandler
from socketserver import TCPServer


from UpdateManager.Core.MetaRelease import (
    Dist,
    MetaReleaseCore,
    MetaReleaseParseError,
)
from UpdateManager.Core.utils import url_downloadable

CURDIR = os.path.dirname(os.path.abspath(__file__))


class SillyProxyRequestHandler(BaseHTTPRequestHandler):
    def do_HEAD(self):
        code = 200
        info = ""
        try:
            f = urlopen(self.path)
            info = f.info()
        except HTTPError as e:
            code = e.code
        s = "HTTP/1.0 %s OK\n%s" % (code, info)
        self.wfile.write(s.encode("UTF-8"))

    # well, good enough
    do_GET = do_HEAD


def get_new_dist(current_release):
    """
    common code to test new dist fetching, get the new dist information
    for hardy+1
    """
    meta = MetaReleaseCore()
    # meta.DEBUG = True
    meta.current_dist_name = current_release
    fake_metarelease = os.path.join(CURDIR, "test-data", "meta-release")
    meta.METARELEASE_URI = "file://%s" % fake_metarelease
    while meta.downloading:
        time.sleep(0.1)
    meta._buildMetaReleaseFile()
    meta.download()
    return meta.new_dist


class TestMetaReleaseCore(unittest.TestCase):
    def setUp(self):
        self.new_dist = None
        self.port = random.randint(1025, 65535)
        self.httpd = TCPServer(("", self.port), SillyProxyRequestHandler)
        self.httpd_process = multiprocessing.Process(
            target=self.httpd.serve_forever
        )
        self.httpd_process.start()

    def tearDown(self):
        self.httpd_process.terminate()
        self.httpd_process.join()
        install_opener(None)

        bin_list = glob.glob(
            os.path.join(CURDIR, "aptroot-changelog/var/cache/apt/*.bin")
        )
        for bin_file in bin_list:
            try:
                os.remove(bin_file)
            except OSError:
                pass

    def testnewdist(self):
        """test that upgrades offer the right upgrade path"""
        for (current, next) in [
            ("dapper", "hardy"),
            ("hardy", "lucid"),
            ("intrepid", "jaunty"),
            ("jaunty", "karmic"),
            ("karmic", "lucid"),
        ]:
            new_dist = get_new_dist(current)
            self.assertEqual(
                next,
                new_dist.name,
                "New dist name for %s is '%s', "
                "but expected '%s''" % (current, new_dist.name, next),
            )

    @unittest.skipUnless(
        url_downloadable("https://ubuntu.com", logging.debug),
        "Could not reach https test site",
    )
    def test_https_url_downloadable(self):
        with EnvironmentVarGuard() as environ:
            logging.debug("no proxy, https address")
            del environ["http_proxy"]
            install_opener(None)
            self.assertTrue(
                url_downloadable("https://ubuntu.com", logging.debug),
                "download https with no proxy failed",
            )

    def test_url_downloadable(self):
        from UpdateManager.Core.utils import url_downloadable

        with EnvironmentVarGuard() as environ:
            # ensure that $no_proxy doesn't prevent us from accessing
            # localhost through proxy
            try:
                del environ["no_proxy"]
            except KeyError:
                pass
            logging.debug("proxy 1")
            environ["http_proxy"] = "http://localhost:%s/" % self.port
            install_opener(None)
            self.assertTrue(
                url_downloadable("http://archive.ubuntu.com", logging.debug),
                "download with proxy %s failed" % environ["http_proxy"],
            )
            logging.debug("proxy 2")
            environ["http_proxy"] = "http://localhost:%s" % self.port
            install_opener(None)
            self.assertTrue(
                url_downloadable("http://archive.ubuntu.com", logging.debug),
                "download with proxy %s failed" % environ["http_proxy"],
            )
            logging.debug("no proxy")
            del environ["http_proxy"]
            install_opener(None)
            self.assertTrue(
                url_downloadable("http://archive.ubuntu.com", logging.debug),
                "download with no proxy failed",
            )

            logging.debug("no proxy, no valid address")
            self.assertFalse(
                url_downloadable(
                    "http://archive.ubuntu.com/xxx", logging.debug
                ),
                "download with no proxy failed",
            )

            logging.debug("proxy, no valid address")
            environ["http_proxy"] = "http://localhost:%s" % self.port
            install_opener(None)
            self.assertFalse(
                url_downloadable(
                    "http://archive.ubuntu.com/xxx", logging.debug
                ),
                "download with no proxy failed",
            )

    def test_get_uri_query_string(self):
        # test with fake data, use a space to test quoting
        d = Dist("xenial", "16.04 LTS", "2016-04-21", True)
        meta = MetaReleaseCore()
        q = meta._get_release_notes_uri_query_string(d)
        self.assertTrue("os%3Dubuntu" in q)
        self.assertTrue("ver%3D16.04%20LTS" in q)

    def test_html_uri_real(self):
        # test parsing of a meta-releaes file from the server
        di = distro_info.UbuntuDistroInfo()
        ltses = [
            supported for supported in di.supported() if di.is_lts(supported)
        ]
        with EnvironmentVarGuard() as environ:
            environ["META_RELEASE_FAKE_CODENAME"] = ltses[-2]
            meta = MetaReleaseCore(forceDownload=True)
            while meta.downloading:
                time.sleep(0.1)
            # it's possible the next LTS is in development
            if meta.new_dist is None:
                meta = MetaReleaseCore(
                    useDevelopmentRelease=True, forceDownload=True
                )
                while meta.downloading:
                    time.sleep(0.1)
            self.assertIsNotNone(meta.new_dist)
            uri = meta.new_dist.releaseNotesHtmlUri
            f = urlopen(uri)
            data = f.read().decode("UTF-8")
            self.assertTrue(len(data) > 0)
            self.assertTrue("<html>" in data)

    @patch("UpdateManager.Core.MetaRelease.MetaReleaseCore.download")
    @patch(
        "UpdateManager.Core.MetaRelease.distro_info.UbuntuDistroInfo.is_lts"
    )
    def test_prompt_lts_ignored_for_non_lts(self, mock_is_lts, mock_download):
        with tempfile.NamedTemporaryFile() as f:
            # for non-LTS, Prompt=lts should not change the METARELEASE_URI
            mock_is_lts.return_value = False
            f.write(
                """
[DEFAULT]
Prompt=lts
""".encode(
                    "utf-8"
                )
            )
            f.flush()
            with patch.object(MetaReleaseCore, "CONF", f.name):
                meta = MetaReleaseCore()
            self.assertNotEqual(meta.METARELEASE_URI, meta.METARELEASE_URI_LTS)

    @patch("UpdateManager.Core.MetaRelease.MetaReleaseCore.download")
    def test_parse_fails_for_all_non_tagfiles(self, mock_download):
        meta = MetaReleaseCore()
        with tempfile.TemporaryFile() as f:
            f.write("random stuff".encode("utf-8"))
            f.seek(0)
            meta.metarelease_information = f
            self.assertRaises(MetaReleaseParseError, meta.parse)

    @patch("UpdateManager.Core.MetaRelease.MetaReleaseCore.download")
    def test_parse_good(self, mock_download):
        meta = MetaReleaseCore()
        meta.current_dist_name = "foo"
        with tempfile.TemporaryFile() as f:
            f.write(
                """Dist: foo
Supported: 1
Date: Thu, 26 Oct 2006 12:00:00 UTC
Version: 1.0

Dist: goo
Supported: 1
Date: Thu, 26 Oct 2016 12:00:00 UTC
Version: 2.0
            """.encode(
                    "utf-8"
                )
            )
            f.seek(0)
            meta.metarelease_information = f
            meta.parse()
            self.assertEqual(meta.upgradable_to.name, "goo")
            self.assertEqual(meta.upgradable_to.version, "2.0")
            self.assertEqual(meta.upgradable_to.supported, True)

    @patch("UpdateManager.Core.MetaRelease.MetaReleaseCore.download")
    def test_parse_next_release_unsupported(self, mock_download):
        # We should jump over an unsupported release. LP: #1497024
        meta = MetaReleaseCore()
        meta.current_dist_name = "foo"
        with tempfile.TemporaryFile() as f:
            f.write(
                """Dist: foo
Supported: 1
Date: Thu, 26 Oct 2006 12:00:00 UTC
Version: 1.0

Dist: goo
Supported: 0
Date: Thu, 26 Oct 2016 12:00:00 UTC
Version: 2.0

Dist: hoo
Supported: 1
Date: Thu, 26 Oct 2026 12:00:00 UTC
Version: 3.0
            """.encode(
                    "utf-8"
                )
            )
            f.seek(0)
            meta.metarelease_information = f
            meta.parse()
            self.assertEqual(meta.upgradable_to.name, "hoo")
            self.assertEqual(meta.upgradable_to.version, "3.0")
            self.assertEqual(meta.upgradable_to.supported, True)

    @patch("UpdateManager.Core.MetaRelease.MetaReleaseCore.download")
    def test_parse_next_release_unsupported_devel(self, mock_download):
        # We should not jump over an unsupported release if we are running in
        # "devel" mode. LP: #1497024
        meta = MetaReleaseCore()
        meta.current_dist_name = "foo"
        meta.useDevelopmentRelease = True
        with tempfile.TemporaryFile() as f:
            f.write(
                """Dist: foo
Supported: 1
Date: Thu, 26 Oct 2006 12:00:00 UTC
Version: 1.0

Dist: goo
Supported: 0
Date: Thu, 26 Oct 2016 12:00:00 UTC
Version: 2.0
            """.encode(
                    "utf-8"
                )
            )
            f.seek(0)
            meta.metarelease_information = f
            meta.parse()
            self.assertEqual(meta.upgradable_to.name, "goo")
            self.assertEqual(meta.upgradable_to.version, "2.0")
            self.assertEqual(meta.upgradable_to.supported, False)

    @patch("builtins.open", new_callable=mock_open)
    @patch("glob.glob")
    @patch("os.path.exists")
    def test_config_parsing(self, mock_exists, mock_glob, mock_file):
        mock_exists.return_value = True
        test_configs = [
            '/etc/update-manager/release-upgrades.d/00-first.cfg',
            '/etc/update-manager/release-upgrades.d/10-second.cfg',
            '/etc/update-manager/release-upgrades.d/20-third.cfg',
        ]
        mock_glob.return_value = test_configs

        file_contents = {
            '/etc/update-manager/release-upgrades':
                '[DEFAULT]\nprompt=lts\n',
            '/etc/update-manager/release-upgrades.d/10-second.cfg':
                'Just some invalid things',
            '/etc/update-manager/release-upgrades.d/20-third.cfg':
                '[DEFAULT]\nprompt=never\n',
            '/etc/update-manager/release-upgrades.d/00-first.cfg':
                '[DEFAULT]\nprompt=lts\n',
        }

        # Configure mock_open to return different content based on filename
        def open_side_effect(filename, *args, **kwargs):
            content = file_contents.get(filename, '')
            return mock_open(read_data=content).return_value

        mock_file.side_effect = open_side_effect

        meta = MetaReleaseCore()
        self.assertEqual(meta.prompt, "never")

        mock_glob.return_value = []
        meta = MetaReleaseCore()
        self.assertEqual(meta.prompt, "lts")


if __name__ == "__main__":
    if len(sys.argv) > 1 and sys.argv[1] == "-v":
        logging.basicConfig(level=logging.DEBUG)
    unittest.main()



# MyCache.py
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
#  Copyright (c) 2004-2008 Canonical
#
#  Author: Michael Vogt <mvo@debian.org>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA

import warnings

warnings.filterwarnings("ignore", "apt API not stable yet", FutureWarning)
import apt
import apt_pkg
import logging
import os

from urllib.error import HTTPError
from urllib.request import urlopen
from urllib.parse import urlsplit
from http.client import BadStatusLine
import socket
import subprocess
import re
import DistUpgrade.DistUpgradeCache
from gettext import gettext as _

try:
    from launchpadlib.launchpad import Launchpad
except ImportError:
    Launchpad = None

SYNAPTIC_PINFILE = "/var/lib/synaptic/preferences"
CHANGELOGS_POOL = "https://changelogs.ubuntu.com/changelogs/pool/"
CHANGELOGS_URI = CHANGELOGS_POOL + "%s/%s/%s/%s_%s/%s"


class HttpsChangelogsUnsupportedError(Exception):
    """https changelogs with credentials are unsupported because of the
    lack of certitifcation validation in urllib2 which allows MITM
    attacks to steal the credentials
    """

    pass


class MyCache(DistUpgrade.DistUpgradeCache.MyCache):

    CHANGELOG_ORIGIN = "Ubuntu"

    def __init__(self, progress, rootdir=None):
        apt.Cache.__init__(self, progress, rootdir)
        # save for later
        self.rootdir = rootdir
        # raise if we have packages in reqreinst state
        # and let the caller deal with that (runs partial upgrade)
        assert len(self.req_reinstall_pkgs) == 0
        # check if the dpkg journal is ok (we need to do that here
        # too because libapt will only do it when it tries to lock
        # the packaging system)
        assert not self._dpkgJournalDirty()
        # init the regular cache
        self._initDepCache()
        self.all_changes = {}
        self.all_news = {}
        self.pro_versions = {}
        # on broken packages, try to fix via saveDistUpgrade()
        if self._depcache.broken_count > 0:
            self.saveDistUpgrade()
        assert (
            self._depcache.broken_count == 0 and self._depcache.del_count == 0
        )
        self.launchpad = None
        # generate versioned_kernel_pkgs_regexp for later use
        apt_versioned_kernel_pkgs = apt_pkg.config.value_list(
            "APT::VersionedKernelPackages"
        )
        if apt_versioned_kernel_pkgs:
            self.versioned_kernel_pkgs_regexp = re.compile(
                "("
                + "|".join(["^" + p for p in apt_versioned_kernel_pkgs])
                + ")"
            )
            running_kernel_version = subprocess.check_output(
                ["uname", "-r"], universal_newlines=True
            ).rstrip()
            self.running_kernel_pkgs_regexp = re.compile(
                "("
                + "|".join(
                    [
                        ("^" + p + ".*" + running_kernel_version)
                        if not p.startswith(".*")
                        else (running_kernel_version + p)
                        for p in apt_versioned_kernel_pkgs
                    ]
                )
                + ")"
            )
        else:
            self.versioned_kernel_pkgs_regexp = None
            self.running_kernel_pkgs_regexp = None

    def _dpkgJournalDirty(self):
        """
        test if the dpkg journal is dirty
        (similar to debSystem::CheckUpdates)
        """
        d = (
            os.path.dirname(apt_pkg.config.find_file("Dir::State::status"))
            + "/updates"
        )
        for f in os.listdir(d):
            if re.match("[0-9]+", f):
                return True
        return False

    def _initDepCache(self):
        # apt_pkg.config.set("Debug::pkgPolicy","1")
        # self.depcache = apt_pkg.GetDepCache(self.cache)
        # self._depcache = apt_pkg.GetDepCache(self._cache)
        self._depcache.read_pinfile()
        if os.path.exists(SYNAPTIC_PINFILE):
            self._depcache.read_pinfile(SYNAPTIC_PINFILE)
        self._depcache.init()

    def clear(self):
        self._initDepCache()

    @property
    def required_download(self):
        """get the size of the packages that are required to download"""
        pm = apt_pkg.PackageManager(self._depcache)
        fetcher = apt_pkg.Acquire()
        pm.get_archives(fetcher, self._list, self._records)
        return fetcher.fetch_needed

    @property
    def install_count(self):
        return self._depcache.inst_count

    def keep_count(self):
        return self._depcache.keep_count

    @property
    def del_count(self):
        return self._depcache.del_count

    def _check_dependencies(self, target, deps):
        """Return True if any of the dependencies in deps match target."""
        # TODO: handle virtual packages
        for dep_or in deps:
            if not dep_or:
                continue
            match = True
            for base_dep in dep_or:
                if (
                    base_dep.name != target.package.shortname
                    or not apt_pkg.check_dep(
                        target.version, base_dep.relation, base_dep.version
                    )
                ):
                    match = False
            if match:
                return True
        return False

    def find_removal_justification(self, pkg):
        target = pkg.installed
        if not target:
            return False
        for cpkg in self:
            candidate = cpkg.candidate
            if candidate is not None:
                if self._check_dependencies(
                    target, candidate.get_dependencies("Conflicts")
                ) and self._check_dependencies(
                    target, candidate.get_dependencies("Replaces")
                ):
                    logging.info(
                        "%s Conflicts/Replaces %s; allowing removal"
                        % (candidate.package.shortname, pkg.shortname)
                    )
                    return True
        return False

    def saveDistUpgrade(self):
        """this functions mimics a upgrade but will never remove anything"""
        # self._apply_dselect_upgrade()
        self._depcache.upgrade(True)
        wouldDelete = self._depcache.del_count
        if wouldDelete > 0:
            deleted_pkgs = [pkg for pkg in self if pkg.marked_delete]
            assert wouldDelete == len(deleted_pkgs)
            for pkg in deleted_pkgs:
                if self.find_removal_justification(pkg):
                    wouldDelete -= 1
        if wouldDelete > 0:
            self.clear()
            assert (
                self._depcache.broken_count == 0
                and self._depcache.del_count == 0
            )
        else:
            assert self._depcache.broken_count == 0
        # self._apply_dselect_upgrade()
        self._depcache.upgrade()
        return wouldDelete

    def _strip_epoch(self, verstr):
        "strip of the epoch"
        vers_no_epoch = verstr.split(":")
        if len(vers_no_epoch) > 1:
            verstr = "".join(vers_no_epoch[1:])
        return verstr

    def _get_changelog_or_news(
        self, name, fname, strict_versioning=False, changelogs_uri=None
    ):
        "helper that fetches the file in question"
        # don't touch the gui in this function, it needs to be thread-safe
        pkg = self[name]

        # get the src package name
        srcpkg = pkg.candidate.source_name

        # assume "main" section
        src_section = "main"
        # use the section of the candidate as a starting point
        section = pkg._pcache._depcache.get_candidate_ver(pkg._pkg).section

        # get the source version
        srcver_epoch = pkg.candidate.source_version
        srcver = self._strip_epoch(srcver_epoch)

        split_section = section.split("/")
        if len(split_section) > 1:
            src_section = split_section[0]

        # lib is handled special
        prefix = srcpkg[0]
        if srcpkg.startswith("lib"):
            prefix = "lib" + srcpkg[3]

        # the changelogs_uri argument overrides the default changelogs_uri,
        # this is useful for e.g. PPAs where we construct the changelogs
        # path differently
        if changelogs_uri:
            uri = changelogs_uri
        else:
            uri = CHANGELOGS_URI % (
                src_section,
                prefix,
                srcpkg,
                srcpkg,
                srcver,
                fname,
            )

        # https uris are not supported when they contain a username/password
        # because the urllib2 https implementation will not check certificates
        # and so its possible to do a man-in-the-middle attack to steal the
        # credentials
        res = urlsplit(uri)
        if res.scheme == "https" and res.username:
            raise HttpsChangelogsUnsupportedError(
                "https locations with username/password are not"
                "supported to fetch changelogs"
            )

        # print("Trying: %s " % uri)
        changelog = urlopen(uri)
        # print(changelog.read())
        # do only get the lines that are new
        alllines = ""
        regexp = "^%s \\((.*)\\)(.*)$" % (re.escape(srcpkg))

        while True:
            line = changelog.readline().decode("UTF-8", "replace")
            if line == "":
                break
            match = re.match(regexp, line)
            if match:
                # strip epoch from installed version
                # and from changelog too
                installed = getattr(pkg.installed, "version", None)
                if installed and ":" in installed:
                    installed = installed.split(":", 1)[1]
                changelogver = match.group(1)
                if changelogver and ":" in changelogver:
                    changelogver = changelogver.split(":", 1)[1]
                # we test for "==" here for changelogs
                # to ensure that the version
                # is actually really in the changelog - if not
                # just display it all, this catches cases like:
                # gcc-defaults with "binver=4.3.1" and srcver=1.76
                #
                # for NEWS.Debian we do require the changelogver > installed
                if strict_versioning:
                    if (
                        installed
                        and apt_pkg.version_compare(changelogver, installed)
                        < 0
                    ):
                        break
                else:
                    if (
                        installed
                        and apt_pkg.version_compare(changelogver, installed)
                        == 0
                    ):
                        break
            alllines = alllines + line
        return alllines

    def _extract_ppa_changelog_uri(self, name):
        """Return the changelog URI from the Launchpad API

        Return None in case of an error.
        """
        if not Launchpad:
            logging.warning(
                "Launchpadlib not available, cannot retrieve PPA changelog"
            )
            return None

        cdt = self[name].candidate
        for uri in cdt.uris:
            if urlsplit(uri).hostname not in (
                "ppa.launchpad.net",
                "ppa.launchpadcontent.net",
            ):
                continue
            match = re.search("http.*/(.*)/(.*)/ubuntu/.*", uri)
            if match is not None:
                user, ppa = match.group(1), match.group(2)
                break
        else:
            logging.error("Unable to find a valid PPA candidate URL.")
            return

        # Login on launchpad if we are not already
        if self.launchpad is None:
            self.launchpad = Launchpad.login_anonymously(
                "update-manager", "production", version="devel"
            )

        archive = self.launchpad.archives.getByReference(
            reference="~%s/ubuntu/%s" % (user, ppa)
        )
        if archive is None:
            logging.error(
                "Unable to retrieve the archive from the Launchpad API."
            )
            return

        spphs = archive.getPublishedSources(
            source_name=cdt.source_name,
            exact_match=True,
            version=cdt.source_version,
        )
        if not spphs:
            logging.error(
                "No published sources were retrieved from the "
                "Launchpad API."
            )
            return

        return spphs[0].changelogUrl()

    def _guess_third_party_changelogs_uri_by_source(self, name):
        pkg = self[name]
        deb_uri = pkg.candidate.uri
        if deb_uri is None:
            return None
        srcrec = pkg.candidate.record.get("Source")
        if not srcrec:
            return None
        # srcpkg can be "apt" or "gcc-default (1.0)"
        srcpkg = srcrec.split("(")[0].strip()
        if "(" in srcrec:
            srcver = srcrec.split("(")[1].rstrip(")")
        else:
            srcver = pkg.candidate.source_version
        base_uri = deb_uri.rpartition("/")[0]
        return base_uri + "/%s_%s.changelog" % (srcpkg, srcver)

    def _guess_third_party_changelogs_uri_by_binary(self, name):
        """guess changelogs uri based on ArchiveURI by replacing .deb
        with .changelog
        """
        # there is always a pkg and a pkg.candidate, no need to add
        # check here
        pkg = self[name]
        deb_uri = pkg.candidate.uri
        if deb_uri:
            return "%s.changelog" % deb_uri.rsplit(".", 1)[0]
        return None

    def get_news_and_changelog(self, name, cb=None, cb_args=None):
        self.get_news(name)
        self.get_changelog(name)
        if cb:
            cb(cb_args)

    def get_news(self, name):
        "get the NEWS.Debian file from the changelogs location"
        try:
            news = self._get_changelog_or_news(name, "NEWS.Debian", True)
        except Exception:
            return
        if news:
            self.all_news[name] = news

    def _fetch_changelog_for_third_party_package(self, name, origins):
        # Special case for PPAs
        changelogs_uri_ppa = None
        for origin in origins:
            if origin.origin.startswith("LP-PPA-"):
                try:
                    changelogs_uri_ppa = self._extract_ppa_changelog_uri(name)
                    break
                except Exception:
                    logging.exception(
                        "Unable to connect to the Launchpad API."
                    )
        # Try non official changelog location
        changelogs_uri_binary = (
            self._guess_third_party_changelogs_uri_by_binary(name)
        )
        changelogs_uri_source = (
            self._guess_third_party_changelogs_uri_by_source(name)
        )
        error_message = ""
        for changelogs_uri in [
            changelogs_uri_ppa,
            changelogs_uri_binary,
            changelogs_uri_source,
        ]:
            if changelogs_uri:
                try:
                    changelog = self._get_changelog_or_news(
                        name, "changelog", False, changelogs_uri
                    )
                    self.all_changes[name] += changelog
                except (HTTPError, HttpsChangelogsUnsupportedError):
                    # no changelogs_uri or 404
                    error_message = _(
                        "This update does not come from a "
                        "source that supports changelogs."
                    )
                except (IOError, BadStatusLine, socket.error):
                    # network errors and others
                    logging.exception("error on changelog fetching")
                    error_message = _(
                        "Failed to download the list of changes. \n"
                        "Please check your Internet connection."
                    )
        self.all_changes[name] += error_message

    # If the machine is not attached to Ubuntu Pro, Update Manager advertises
    # the upgrades that would be available if it were attached.
    # As that is unbeknownst to Apt, we need this map to show the correct
    # version of each upgradable-if-pro-subscribed package.
    def create_pro_cache(self, pro_pkgs):
        for (name, version, _a, _a) in pro_pkgs:
            self.pro_versions[name] = version

    def get_changelog(self, name):
        "get the changelog file from the changelog location"
        origins = self[name].candidate.origins
        version = self.pro_versions.get(name, self[name].candidate.version)
        self.all_changes[name] = _(
            "Changes for %s versions:\n"
            "Installed version: %s\n"
            "Available version: %s\n\n"
        ) % (
            name,
            getattr(self[name].installed, "version", None),
            version,
        )
        if self.CHANGELOG_ORIGIN not in [o.origin for o in origins]:
            self._fetch_changelog_for_third_party_package(name, origins)
            return
        # fixup epoch handling version
        srcpkg = self[name].candidate.source_name
        srcver_epoch = self[name].candidate.source_version.replace(":", "%3A")
        try:
            changelog = self._get_changelog_or_news(name, "changelog")
            if len(changelog) == 0:
                changelog = _(
                    "The changelog does not contain any relevant changes.\n\n"
                    "Please use http://launchpad.net/ubuntu/+source/"
                    "%s/%s/+changelog\n"
                    "until the changes become available or try again later."
                ) % (srcpkg, srcver_epoch)
        except HTTPError:
            changelog = _(
                "The list of changes is not available yet.\n\n"
                "Please use http://launchpad.net/ubuntu/+source/"
                "%s/%s/+changelog\n"
                "until the changes become available or try again later."
            ) % (srcpkg, srcver_epoch)
        except (IOError, BadStatusLine, socket.error) as e:
            print("caught exception: ", e)
            changelog = _(
                "Failed to download the list of changes. \n"
                "Please check your Internet connection."
            )
        self.all_changes[name] += changelog



# Copyright (C) 2008-2012  Canonical, Ltd.
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, version 3 of the License.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

"""A test plugin."""

__metaclass__ = type
__all__ = ["CharlieCruft", "CharliePlugin"]

from janitor.plugincore.cruft import Cruft
from janitor.plugincore.plugin import Plugin


class CharlieCruft(Cruft):
    def __init__(self, app):
        self.app = app

    def get_shortname(self):
        return "Charlie"

    def cleanup(self):
        # Tell the app we're cleaning up this cruft.
        self.app.notifications.append((self, "cruft"))


class CharliePlugin(Plugin):
    def get_cruft(self):
        yield CharlieCruft(self.app)

    def post_cleanup(self):
        self.app.notifications.append((self, "post"))



#!/usr/bin/python3
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-

import logging
import mock
import sys
import unittest

from UpdateManager.Core import utils


class TestUtils(unittest.TestCase):
    def test_humanize_size(self):
        # humanize size is a bit funny, it rounds up to kB as the meaningful
        # unit for users
        self.assertEqual(utils.humanize_size(1000), "1 kB")
        self.assertEqual(utils.humanize_size(10), "1 kB")
        self.assertEqual(utils.humanize_size(1200), "2 kB")
        # but not for MB as well
        self.assertEqual(utils.humanize_size(1200 * 1000), "1.2 MB")
        self.assertEqual(utils.humanize_size(1478 * 1000), "1.5 MB")
        # and we don't go to Gb  just yet (as its not really needed
        # in a upgrade context most of the time
        self.assertEqual(utils.humanize_size(1000 * 1000 * 1000), "1000.0 MB")

    def test_is_child_of_process_name(self):
        data = "1 (systemd) S 0 1 1 0 -1 4194560 255183 71659081 87 18403 \
816 737 430168 107409 20 0 1 0 2 218931200 1882 18446744073709551615 1 1 0 0 \
0 0 671173123 4096 1260 0 0 0 17 3 0 0 469245113 0 258124 0 0 0 0 0 0 0 0"
        with mock.patch(
            "builtins.open", mock.mock_open(read_data=data)
        ) as mock_file:
            assert open("/proc/1/stat").read() == data
            mock_file.assert_called_with("/proc/1/stat")
            self.assertTrue(
                utils.is_child_of_process_name("init")
                or utils.is_child_of_process_name("systemd")
            )
            self.assertFalse(utils.is_child_of_process_name("mvo"))

    def test_is_port_listening(self):
        from UpdateManager.Core.utils import is_port_already_listening

        data = "9: 00000000:0016 00000000:0000 0A 00000000:00000000 \
00:00000000 00000000     0        0 11366514 1 0000000000000000 100 \
0 0 10 0"
        with mock.patch(
            "builtins.open", mock.mock_open(read_data=data)
        ) as mock_file:
            assert open("/proc/net/tcp").readlines() == [data]
            mock_file.assert_called_with("/proc/net/tcp")
            self.assertTrue(is_port_already_listening(22))

    def test_strip_auth_from_source_entry(self):
        from aptsources.sourceslist import SourceEntry

        # entry with PW
        s = SourceEntry("deb http://user:pass@some-ppa/ ubuntu main")
        self.assertTrue(
            "user" not in utils.get_string_with_no_auth_from_source_entry(s)
        )
        self.assertTrue(
            "pass" not in utils.get_string_with_no_auth_from_source_entry(s)
        )
        self.assertEqual(
            utils.get_string_with_no_auth_from_source_entry(s),
            "deb http://hidden-u:hidden-p@some-ppa/ ubuntu main",
        )
        # no pw
        s = SourceEntry("deb http://some-ppa/ ubuntu main")
        self.assertEqual(
            utils.get_string_with_no_auth_from_source_entry(s),
            "deb http://some-ppa/ ubuntu main",
        )

    @mock.patch("UpdateManager.Core.utils._load_meta_pkg_list")
    def test_flavor_package_ubuntu_first(self, mock_load):
        cache = {
            "ubuntu-desktop": mock.MagicMock(),
            "other-desktop": mock.MagicMock(),
        }
        cache["ubuntu-desktop"].is_installed = True
        cache["other-desktop"].is_installed = True
        mock_load.return_value = ["other-desktop"]
        self.assertEqual(
            utils.get_ubuntu_flavor_package(cache=cache), "ubuntu-desktop"
        )

    @mock.patch("UpdateManager.Core.utils._load_meta_pkg_list")
    def test_flavor_package_match(self, mock_load):
        cache = {
            "a": mock.MagicMock(),
            "b": mock.MagicMock(),
            "c": mock.MagicMock(),
        }
        cache["a"].is_installed = True
        cache["b"].is_installed = True
        cache["c"].is_installed = True
        mock_load.return_value = ["c", "a", "b"]
        # Must pick alphabetically first
        self.assertEqual(utils.get_ubuntu_flavor_package(cache=cache), "a")

    def test_flavor_package_default(self):
        self.assertEqual(
            utils.get_ubuntu_flavor_package(cache={}), "ubuntu-desktop"
        )

    def test_flavor_default(self):
        self.assertEqual(utils.get_ubuntu_flavor(cache={}), "ubuntu")

    @mock.patch("UpdateManager.Core.utils.get_ubuntu_flavor_package")
    def test_flavor_simple(self, mock_package):
        mock_package.return_value = "d"
        self.assertEqual(utils.get_ubuntu_flavor(), "d")

    @mock.patch("UpdateManager.Core.utils.get_ubuntu_flavor_package")
    def test_flavor_chop(self, mock_package):
        mock_package.return_value = "d-pkg"
        self.assertEqual(utils.get_ubuntu_flavor(), "d")

    @mock.patch("UpdateManager.Core.utils.get_ubuntu_flavor_package")
    def test_flavor_name_desktop(self, mock_package):
        mock_package.return_value = "something-desktop"
        self.assertEqual(utils.get_ubuntu_flavor_name(), "Something")

    @mock.patch("UpdateManager.Core.utils.get_ubuntu_flavor_package")
    def test_flavor_name_netbook(self, mock_package):
        mock_package.return_value = "something-netbook"
        self.assertEqual(utils.get_ubuntu_flavor_name(), "Something")

    @mock.patch("UpdateManager.Core.utils.get_ubuntu_flavor_package")
    def test_flavor_name_studio(self, mock_package):
        mock_package.return_value = "ubuntustudio-desktop"
        self.assertEqual(utils.get_ubuntu_flavor_name(), "Ubuntu Studio")


if __name__ == "__main__":
    if len(sys.argv) > 1 and sys.argv[1] == "-v":
        logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/env python3
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
# (c) 2005-2012 Canonical, GPL
# (C) 2008-2009 Sebastian Heinlein <devel@glatzor.de>

from gi.repository import Gtk

from aptdaemon import client, errors
from defer import inline_callbacks
from aptdaemon.gtk3widgets import (
    AptCancelButton,
    AptConfigFileConflictDialog,
    AptDetailsExpander,
    AptMediumRequiredDialog,
    AptProgressBar,
)
from aptdaemon.enums import (
    EXIT_SUCCESS,
    EXIT_FAILED,
    STATUS_COMMITTING,
    STATUS_DOWNLOADING,
    STATUS_DOWNLOADING_REPO,
    STATUS_FINISHED,
    get_error_description_from_enum,
    get_error_string_from_enum,
    get_status_string_from_enum,
)

from UpdateManager.backend import InstallBackend
from UpdateManager.UnitySupport import UnitySupport
from UpdateManager.Dialogs import BuilderDialog

from gettext import gettext as _

import dbus
import os


class UpdateManagerExpander(AptDetailsExpander):
    """An AptDetailsExpander which can be used with multiple terminals.

    The default AptDetailsExpander will shrink/hide when its transaction
    finishes. But here we want to support "chaining" transactions. So we
    override the status-changed handler to only do that when we are
    running the final transaction."""

    def __init__(self, transaction, terminal=True, final=False):
        super().__init__(transaction, terminal)
        self.final = final

    def _on_status_changed(self, trans, status):
        if status in (STATUS_DOWNLOADING, STATUS_DOWNLOADING_REPO):
            self.set_sensitive(True)
            self.download_scrolled.set_vexpand(True)
            self.download_scrolled.show()
            if self.terminal:
                self.terminal.hide()
        elif status == STATUS_COMMITTING:
            self.download_scrolled.hide()
            if self.terminal:
                self.terminal.show()
                self.set_sensitive(True)
            elif self.final:
                self.set_expanded(False)
                self.set_sensitive(False)
        elif self.final and status == STATUS_FINISHED:
            self.download_scrolled.hide()
            if self.terminal:
                self.terminal.hide()
            self.set_sensitive(False)
            self.set_expanded(False)


class AptStackedProgressBar(Gtk.ProgressBar):
    """A GtkProgressBar which represents the state of many aptdaemon
    transactions.

    aptdaemon provides AptProgressBar for the state of *one* transaction to
    be represented in a progress bar. This widget creates one of those per
    containing transaction, and scales its progress to the given ratio, so
    one progress bar can show the state of many transactions."""

    def __init__(self, unity):
        self.current_max_progress = 0
        self.progress_bars = []
        self.unity = unity

        super().__init__()

    def add_transaction(self, trans, max_progress):
        assert 0 <= max_progress <= 1

        progress = AptProgressBar(trans)
        self.progress_bars.append(progress)
        progress.min = self.current_max_progress
        self.current_max_progress += max_progress

        if self.current_max_progress > 1:
            self.current_max_progress = 1

        progress.max = self.current_max_progress
        progress.connect("notify::fraction", self._update_progress)
        progress.connect("notify::text", self._update_text)

    def _update_progress(self, inner_progress, data):
        delta = inner_progress.max - inner_progress.min
        position_in_delta = delta * inner_progress.get_fraction()
        new_progress = inner_progress.min + position_in_delta
        self.set_fraction(new_progress)
        self.unity.set_progress(new_progress * 100)

    def _update_text(self, inner_progress, data):
        self.set_text(inner_progress.get_text())


class InstallBackendAptdaemon(InstallBackend, BuilderDialog):
    """Makes use of aptdaemon to refresh the cache and to install updates."""

    def __init__(self, window_main, action):
        InstallBackend.__init__(self, window_main, action)
        ui_path = os.path.join(
            window_main.datadir, "gtkbuilder/UpdateProgress.ui"
        )
        BuilderDialog.__init__(
            self, window_main, ui_path, "pane_update_progress"
        )

        self.client = client.AptClient()
        self.unity = UnitySupport()
        self._expanded_size = None
        self.button_cancel = None
        self.trans_failed_msg = None
        self.progressbar = None
        self._active_transaction = None
        self._expander = None

    def close(self):
        if self.button_cancel and self.button_cancel.get_sensitive():
            try:
                self.button_cancel.clicked()
            except Exception:
                # there is not much left to do if the transaction can't be
                # canceled
                pass
            return True
        else:
            return False

    @inline_callbacks
    def update(self):
        """Refresh the package list"""
        try:
            trans = yield self.client.update_cache(defer=True)
            yield self._show_transaction(
                trans, self.ACTION_UPDATE, _("Checking for updates…"), False
            )
        except errors.NotAuthorizedError:
            self._action_done(
                self.ACTION_UPDATE,
                authorized=False,
                success=False,
                error_string=None,
                error_desc=None,
            )
        except Exception:
            self._action_done(
                self.ACTION_UPDATE,
                authorized=True,
                success=False,
                error_string=None,
                error_desc=None,
            )
            raise

    def _show_transaction_error(self, trans, action):
        error_string = get_error_string_from_enum(trans.error.code)
        error_desc = get_error_description_from_enum(trans.error.code)
        if self.trans_failed_msg:
            trans_failed = True
            error_desc = error_desc + "\n" + self.trans_failed_msg
        else:
            trans_failed = None
        self._action_done(
            action,
            authorized=True,
            success=False,
            error_string=error_string,
            error_desc=error_desc,
            trans_failed=trans_failed,
        )

    def _update_next_package(self, trans, status, action):
        if status == EXIT_FAILED:
            self._show_transaction_error(trans, action)
            return
        self._apt_update_oem()

    @inline_callbacks
    def _apt_update_oem(self):
        assert self._oem_packages_to_update
        elem = self._oem_packages_to_update.pop()
        sources_list_file = f"/etc/apt/sources.list.d/{elem}.list"

        try:
            if os.path.exists(sources_list_file):
                trans = yield self.client.update_cache(
                    sources_list=sources_list_file
                )
                if self._oem_packages_to_update:
                    finished_handler = self._update_next_package
                else:
                    finished_handler = self._on_finished

                yield self._show_transaction(
                    trans,
                    self.ACTION_PRE_INSTALL,
                    _("Installing updates…"),
                    True,
                    on_finished_handler=finished_handler,
                    progress_bar_max=0.1 / self._len_oem_updates,
                )
        except errors.NotAuthorizedError:
            self._action_done(
                self.ACTION_PRE_INSTALL,
                authorized=False,
                success=False,
                error_string=None,
                error_desc=None,
            )
        except errors.TransactionFailed as e:
            self.trans_failed_msg = str(e)
        except dbus.DBusException as e:
            if e.get_dbus_name() != "org.freedesktop.DBus.Error.NoReply":
                raise
            self._action_done(
                self.ACTION_PRE_INSTALL,
                authorized=False,
                success=False,
                error_string=None,
                error_desc=None,
            )
        except Exception:
            self._action_done(
                self.ACTION_PRE_INSTALL,
                authorized=True,
                success=False,
                error_string=None,
                error_desc=None,
            )
            raise

    def _update_oem(self, trans, status, action):
        # This is the "finished" handler of installing an oem metapackage
        # What we do now is:
        #  1. update_cache() for the new sources.lists only

        if status == EXIT_FAILED:
            self._show_transaction_error(trans, action)
            return

        (install, _, _, _, _, _) = trans.packages

        self._oem_packages_to_update = set(install)
        self._len_oem_updates = len(install)

        self._apt_update_oem()

    @inline_callbacks
    def commit_oem(self, pkgs_install_oem, pkgs_upgrade_oem):
        self.all_oem_packages = set(pkgs_install_oem) | set(pkgs_upgrade_oem)
        # Nothing to do? Go to the regular updates.
        try:
            if not pkgs_install_oem and not pkgs_upgrade_oem:
                self._action_done(
                    self.ACTION_PRE_INSTALL,
                    authorized=True,
                    success=True,
                    error_string=None,
                    error_desc=None,
                    trans_failed=None,
                )
                return

            if pkgs_install_oem:
                trans = yield self.client.install_packages(
                    pkgs_install_oem, defer=True
                )
                yield self._show_transaction(
                    trans,
                    self.ACTION_PRE_INSTALL,
                    _("Installing updates…"),
                    True,
                    on_finished_handler=self._update_oem,
                    progress_bar_max=0.1,
                )
        except errors.NotAuthorizedError:
            self._action_done(
                self.ACTION_PRE_INSTALL,
                authorized=False,
                success=False,
                error_string=None,
                error_desc=None,
            )
        except errors.TransactionFailed as e:
            self.trans_failed_msg = str(e)
        except dbus.DBusException as e:
            if e.get_dbus_name() != "org.freedesktop.DBus.Error.NoReply":
                raise
            self._action_done(
                self.ACTION_PRE_INSTALL,
                authorized=False,
                success=False,
                error_string=None,
                error_desc=None,
            )
        except Exception:
            self._action_done(
                self.ACTION_PRE_INSTALL,
                authorized=True,
                success=False,
                error_string=None,
                error_desc=None,
            )
            raise

    @inline_callbacks
    def commit(self, pkgs_install, pkgs_upgrade, pkgs_remove):
        """Commit a list of package adds and removes"""
        try:
            reinstall = purge = downgrade = []
            trans = yield self.client.commit_packages(
                pkgs_install,
                reinstall,
                pkgs_remove,
                purge,
                pkgs_upgrade,
                downgrade,
                defer=True,
            )
            yield self._show_transaction(
                trans, self.ACTION_INSTALL, _("Installing updates…"), True
            )
        except errors.NotAuthorizedError:
            self._action_done(
                self.ACTION_INSTALL,
                authorized=False,
                success=False,
                error_string=None,
                error_desc=None,
            )
        except errors.TransactionFailed as e:
            self.trans_failed_msg = str(e)
        except dbus.DBusException as e:
            # print(e, e.get_dbus_name())
            if e.get_dbus_name() != "org.freedesktop.DBus.Error.NoReply":
                raise
            self._action_done(
                self.ACTION_INSTALL,
                authorized=False,
                success=False,
                error_string=None,
                error_desc=None,
            )
        except Exception:
            self._action_done(
                self.ACTION_INSTALL,
                authorized=True,
                success=False,
                error_string=None,
                error_desc=None,
            )
            raise

    def _on_details_changed(self, trans, details, label_details):
        label_details.set_label(details)

    def _on_status_changed(self, trans, status, label_details):
        label_details.set_label(get_status_string_from_enum(status))
        # Also resize the window if we switch from download details to
        # the terminal window
        if (
            status == STATUS_COMMITTING
            and self._expander
            and self._expander.terminal.get_visible()
        ):
            self._resize_to_show_details(self._expander)

    @inline_callbacks
    def _show_transaction(
        self,
        trans,
        action,
        header,
        show_details,
        progress_bar_max=1,
        on_finished_handler=None,
    ):

        if on_finished_handler is None:
            on_finished_handler = self._on_finished

        self.label_header.set_label(header)
        self.label_header.get_accessible().emit("notification", header, 0)

        if not self.progressbar:
            self.progressbar = AptStackedProgressBar(self.unity)
            self.progressbar.show()
            self.progressbar_slot.add(self.progressbar)

        self.progressbar.add_transaction(trans, progress_bar_max)

        if self.button_cancel:
            self.button_cancel.set_transaction(trans)
        else:
            self.button_cancel = AptCancelButton(trans)
            self.button_cancel.show()
            self.button_cancel_slot.add(self.button_cancel)

        if action == self.ACTION_UPDATE:
            self.button_cancel.set_label(Gtk.STOCK_STOP)

        if show_details:
            if not self._expander:
                self._expander = UpdateManagerExpander(trans)
                self._expander.set_vexpand(True)
                self._expander.set_hexpand(True)
                self._expander.show_all()
                self._expander.connect("notify::expanded", self._on_expanded)
                self.expander_slot.add(self._expander)
                self.expander_slot.show()
            else:
                self._expander.set_transaction(trans)
            self._expander.final = action != self.ACTION_PRE_INSTALL
        elif self._expander:
            self._expander_slot.hide()

        trans.connect(
            "status-details-changed",
            self._on_details_changed,
            self.label_details,
        )
        trans.connect(
            "status-changed", self._on_status_changed, self.label_details
        )
        trans.connect("finished", on_finished_handler, action)
        trans.connect("medium-required", self._on_medium_required)
        trans.connect("config-file-conflict", self._on_config_file_conflict)

        yield trans.set_debconf_frontend("gnome")
        yield trans.run()

    def _on_expanded(self, expander, param):
        # Make the dialog resizable if the expander is expanded
        # try to restore a previous size
        if not expander.get_expanded():
            self._expanded_size = (
                expander.terminal.get_visible(),
                self.window_main.get_size(),
            )
            self.window_main.end_user_resizable()
        elif self._expanded_size:
            term_visible, (stored_width, stored_height) = self._expanded_size
            # Check if the stored size was for the download details or
            # the terminal widget
            if term_visible != expander.terminal.get_visible():
                # The stored size was for the download details, so we need
                # get a new size for the terminal widget
                self._resize_to_show_details(expander)
            else:
                self.window_main.begin_user_resizable(
                    stored_width, stored_height
                )
        else:
            self._resize_to_show_details(expander)

    def _resize_to_show_details(self, expander):
        """Resize the window to show the expanded details.

        Unfortunately the expander only expands to the preferred size of the
        child widget (e.g showing all 80x24 chars of the Vte terminal) if
        the window is rendered the first time and the terminal is also visible.
        If the expander is expanded afterwards the window won't change its
        size anymore. So we have to do this manually. See LP#840942
        """
        if expander.get_expanded():
            win_width, win_height = self.window_main.get_size()
            exp_width = expander.get_allocation().width
            exp_height = expander.get_allocation().height
            if expander.terminal.get_visible():
                terminal_width = expander.terminal.get_char_width() * 80
                terminal_height = expander.terminal.get_char_height() * 24
                new_width = terminal_width - exp_width + win_width
                new_height = terminal_height - exp_height + win_height
            else:
                new_width = win_width + 100
                new_height = win_height + 200
            self.window_main.begin_user_resizable(new_width, new_height)

    def _on_medium_required(self, transaction, medium, drive):
        dialog = AptMediumRequiredDialog(medium, drive, self.window_main)
        res = dialog.run()
        dialog.hide()
        if res == Gtk.ResponseType.OK:
            transaction.provide_medium(medium)
        else:
            transaction.cancel()

    def _on_config_file_conflict(self, transaction, old, new):
        dialog = AptConfigFileConflictDialog(old, new, self.window_main)
        res = dialog.run()
        dialog.hide()
        if res == Gtk.ResponseType.YES:
            transaction.resolve_config_file_conflict(old, "replace")
        else:
            transaction.resolve_config_file_conflict(old, "keep")

    def _on_finished(self, trans, status, action):
        error_string = None
        error_desc = None
        trans_failed = False
        if status == EXIT_FAILED:
            error_string = get_error_string_from_enum(trans.error.code)
            error_desc = get_error_description_from_enum(trans.error.code)
            if self.trans_failed_msg:
                trans_failed = True
                error_desc = error_desc + "\n" + self.trans_failed_msg
        # tell unity to hide the progress again
        self.unity.set_progress(-1)
        is_success = status == EXIT_SUCCESS
        try:
            self._action_done(
                action,
                authorized=True,
                success=is_success,
                error_string=error_string,
                error_desc=error_desc,
                trans_failed=trans_failed,
            )
        except TypeError:
            # this module used to be be lazily imported and in older code
            # trans_failed= is not accepted
            # TODO: this workaround can be dropped in Ubuntu 20.10
            self._action_done(
                action,
                authorized=True,
                success=is_success,
                error_string=error_string,
                error_desc=error_desc,
            )


if __name__ == "__main__":
    import mock

    options = mock.Mock()
    data_dir = "/usr/share/update-manager"

    from UpdateManager.UpdateManager import UpdateManager

    app = UpdateManager(data_dir, options)

    b = InstallBackendAptdaemon(app, None)
    b.commit(["2vcard"], [], [])
    Gtk.main()



# utils.py
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
#  Copyright (c) 2011 Canonical
#
#  Author:  Alex Chiang <achiang@canonical.com>
#           Michael Vogt <michael.vogt@ubuntu.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA

import dbus
import sys


class ModemManagerHelper(object):

    # data taken from
    #  http://projects.gnome.org/NetworkManager/developers/mm-spec-04.html
    MM_DBUS_IFACE = "org.freedesktop.ModemManager"
    MM_DBUS_IFACE_MODEM = MM_DBUS_IFACE + ".Modem"

    # MM_MODEM_TYPE
    MM_MODEM_TYPE_GSM = 1
    MM_MODEM_TYPE_CDMA = 2

    # GSM
    # Not registered, not searching for new operator to register.
    MM_MODEM_GSM_NETWORK_REG_STATUS_IDLE = 0
    # Registered on home network.
    MM_MODEM_GSM_NETWORK_REG_STATUS_HOME = 1
    # Not registered, searching for new operator to register with.
    MM_MODEM_GSM_NETWORK_REG_STATUS_SEARCHING = 2
    # Registration denied.
    MM_MODEM_GSM_NETWORK_REG_STATUS_DENIED = 3
    # Unknown registration status.
    MM_MODEM_GSM_NETWORK_REG_STATUS_UNKNOWN = 4
    # Registered on a roaming network.
    MM_MODEM_GSM_NETWORK_REG_STATUS_ROAMING = 5

    # CDMA
    # Registration status is unknown or the device is not registered.
    MM_MODEM_CDMA_REGISTRATION_STATE_UNKNOWN = 0
    # Registered, but roaming status is unknown or cannot be provided
    # by the device. The device may or may not be roaming.
    MM_MODEM_CDMA_REGISTRATION_STATE_REGISTERED = 1
    #     Currently registered on the home network.
    MM_MODEM_CDMA_REGISTRATION_STATE_HOME = 2
    #     Currently registered on a roaming network.
    MM_MODEM_CDMA_REGISTRATION_STATE_ROAMING = 3

    def __init__(self):
        self.bus = dbus.SystemBus()
        self.proxy = self.bus.get_object(
            "org.freedesktop.ModemManager", "/org/freedesktop/ModemManager"
        )
        modem_manager = dbus.Interface(self.proxy, self.MM_DBUS_IFACE)
        self.modems = modem_manager.EnumerateDevices()

    @staticmethod
    def get_dbus_property(proxy, interface, property):
        props = dbus.Interface(proxy, "org.freedesktop.DBus.Properties")
        property = props.Get(interface, property)
        return property

    def is_gsm_roaming(self):
        for m in self.modems:
            dev = self.bus.get_object(self.MM_DBUS_IFACE, m)
            type = self.get_dbus_property(
                dev, self.MM_DBUS_IFACE_MODEM, "Type"
            )
            if type != self.MM_MODEM_TYPE_GSM:
                continue
            net = dbus.Interface(
                dev, self.MM_DBUS_IFACE_MODEM + ".Gsm.Network"
            )
            reg = net.GetRegistrationInfo()
            # Be conservative about roaming. If registration unknown,
            # assume yes.
            # MM_MODEM_GSM_NETWORK_REG_STATUS
            if reg[0] in (
                self.MM_MODEM_GSM_NETWORK_REG_STATUS_UNKNOWN,
                self.MM_MODEM_GSM_NETWORK_REG_STATUS_ROAMING,
            ):
                return True
        return False

    def is_cdma_roaming(self):
        for m in self.modems:
            dev = self.bus.get_object(self.MM_DBUS_IFACE, m)
            type = self.get_dbus_property(
                dev, self.MM_DBUS_IFACE_MODEM, "Type"
            )
            if type != self.MM_MODEM_TYPE_CDMA:
                continue
            cdma = dbus.Interface(dev, self.MM_DBUS_IFACE_MODEM + ".Cdma")
            (cmda_1x, evdo) = cdma.GetRegistrationState()
            # Be conservative about roaming. If registration unknown,
            # assume yes.
            # MM_MODEM_CDMA_REGISTRATION_STATE
            roaming_states = (
                self.MM_MODEM_CDMA_REGISTRATION_STATE_REGISTERED,
                self.MM_MODEM_CDMA_REGISTRATION_STATE_ROAMING,
            )
            # evdo trumps cmda_1x (thanks to Mathieu Trudel-Lapierre)
            if evdo in roaming_states:
                return True
            elif cmda_1x in roaming_states:
                return True
        return False


class NetworkManagerHelper(object):
    NM_DBUS_IFACE = "org.freedesktop.NetworkManager"

    # connection states
    # Old enum values are for NM 0.7

    # The NetworkManager daemon is in an unknown state.
    NM_STATE_UNKNOWN = 0
    # The NetworkManager daemon is connecting a device.
    NM_STATE_CONNECTING_OLD = 2
    NM_STATE_CONNECTING = 40
    NM_STATE_CONNECTING_LIST = [NM_STATE_CONNECTING_OLD, NM_STATE_CONNECTING]
    # The NetworkManager daemon is connected.
    NM_STATE_CONNECTED_OLD = 3
    NM_STATE_CONNECTED_LOCAL = 50
    NM_STATE_CONNECTED_SITE = 60
    NM_STATE_CONNECTED_GLOBAL = 70
    NM_STATE_CONNECTED_LIST = [
        NM_STATE_CONNECTED_OLD,
        NM_STATE_CONNECTED_LOCAL,
        NM_STATE_CONNECTED_SITE,
        NM_STATE_CONNECTED_GLOBAL,
    ]

    # The device type is unknown.
    NM_DEVICE_TYPE_UNKNOWN = 0
    # The device is wired Ethernet device.
    NM_DEVICE_TYPE_ETHERNET = 1
    # The device is an 802.11 WiFi device.
    NM_DEVICE_TYPE_WIFI = 2
    # The device is a GSM-based cellular WAN device.
    NM_DEVICE_TYPE_GSM = 3
    # The device is a CDMA/IS-95-based cellular WAN device.
    NM_DEVICE_TYPE_CDMA = 4

    def __init__(self):
        self.bus = dbus.SystemBus()
        self.proxy = self.bus.get_object(
            "org.freedesktop.NetworkManager", "/org/freedesktop/NetworkManager"
        )

    @staticmethod
    def get_dbus_property(proxy, interface, property):
        props = dbus.Interface(proxy, "org.freedesktop.DBus.Properties")
        property = props.Get(interface, property)
        return property

    def is_active_connection_gsm_or_cdma(self):
        res = False
        actives = self.get_dbus_property(
            self.proxy, self.NM_DBUS_IFACE, "ActiveConnections"
        )
        for a in actives:
            active = self.bus.get_object(self.NM_DBUS_IFACE, a)
            default_route = self.get_dbus_property(
                active, self.NM_DBUS_IFACE + ".Connection.Active", "Default"
            )
            if not default_route:
                continue
            devs = self.get_dbus_property(
                active, self.NM_DBUS_IFACE + ".Connection.Active", "Devices"
            )
            for d in devs:
                dev = self.bus.get_object(self.NM_DBUS_IFACE, d)
                type = self.get_dbus_property(
                    dev, self.NM_DBUS_IFACE + ".Device", "DeviceType"
                )
                if type == self.NM_DEVICE_TYPE_GSM:
                    return True
                elif type == self.NM_DEVICE_TYPE_CDMA:
                    return True
                else:
                    continue
        return res

    def is_active_connection_gsm_or_cdma_roaming(self):
        res = False
        if self.is_active_connection_gsm_or_cdma():
            mmhelper = ModemManagerHelper()
            res |= mmhelper.is_gsm_roaming()
            res |= mmhelper.is_cdma_roaming()
        return res


if __name__ == "__main__":

    # test code
    if sys.argv[1:] and sys.argv[1] == "--test":
        mmhelper = ModemManagerHelper()
        print("is_gsm_roaming", mmhelper.is_gsm_roaming())
        print("is_cdma_romaing", mmhelper.is_cdma_roaming())

    # roaming?
    nmhelper = NetworkManagerHelper()
    is_roaming = nmhelper.is_active_connection_gsm_or_cdma_roaming()
    print("roam: ", is_roaming)
    if is_roaming:
        sys.exit(1)
    sys.exit(0)



# Copyright (C) 2008-2012  Canonical, Ltd.
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, version 3 of the License.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

__metaclass__ = type
__all__ = ["MissingPackageCruftTests"]


import unittest

from janitor.plugincore.core.missing_package_cruft import MissingPackageCruft
from janitor.plugincore.testing.helpers import MockAptPackage


class MissingPackageCruftTests(unittest.TestCase):
    def setUp(self):
        self.pkg = MockAptPackage()
        self.cruft = MissingPackageCruft(self.pkg)

    def test_prefix(self):
        self.assertEqual(self.cruft.get_prefix(), "install-deb")
        self.assertEqual(self.cruft.prefix, "install-deb")

    def test_prefix_description(self):
        self.assertTrue("Install" in self.cruft.get_prefix_description())
        self.assertTrue("Install" in self.cruft.prefix_description)

    def test_shortname(self):
        self.assertEqual(self.cruft.get_shortname(), "name")
        self.assertEqual(self.cruft.shortname, "name")

    def test_name(self):
        self.assertEqual(self.cruft.get_name(), "install-deb:name")
        self.assertEqual(self.cruft.name, "install-deb:name")

    def test_description(self):
        self.assertTrue("name" in self.cruft.get_description())
        self.assertTrue("name" in self.cruft.description)

    def test_explicit_description(self):
        pkg = MissingPackageCruft(self.pkg, "foo")
        self.assertEqual(pkg.get_description(), "foo")
        self.assertEqual(pkg.description, "foo")

    def test_cleanup(self):
        self.cruft.cleanup()
        self.assertTrue(self.pkg.installed)



#!/usr/bin/python3
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-

import apt
import logging
import os
import sys
import unittest
import glob

from urllib.error import HTTPError
from UpdateManager.Core.MyCache import MyCache

CURDIR = os.path.dirname(os.path.abspath(__file__))


class TestChangelogs(unittest.TestCase):
    def setUp(self):
        # Whenever a test will initialize apt_pkg, we must set the architecture
        # to amd64, because our various test files assume that.  Even if this
        # test doesn't use those files, apt_pkg is only initialized once across
        # tests, so we must be consistent.
        real_arch = apt.apt_pkg.config.find("APT::Architecture")
        apt.apt_pkg.config.set("APT::Architecture", "amd64")
        self.addCleanup(
            lambda: apt.apt_pkg.config.set("APT::Architecture", real_arch)
        )

        self.aptroot = os.path.join(CURDIR, "aptroot-changelog")

        self.cache = MyCache(
            apt.progress.base.OpProgress(), rootdir=self.aptroot
        )
        self.cache.open()

    def tearDown(self):
        bin_list = glob.glob(os.path.join(self.aptroot, "var/cache/apt/*.bin"))
        for bin_file in bin_list:
            try:
                os.remove(bin_file)
            except OSError:
                pass

    def test_get_changelogs_uri(self):
        pkgname = "gcc"
        # test binary changelogs
        uri = self.cache._guess_third_party_changelogs_uri_by_binary(pkgname)
        pkg = self.cache[pkgname]
        self.assertEqual(uri, pkg.candidate.uri.replace(".deb", ".changelog"))
        # test source changelogs
        uri = self.cache._guess_third_party_changelogs_uri_by_source(pkgname)
        self.assertTrue("gcc-defaults_" in uri)
        self.assertTrue(uri.endswith(".changelog"))
        # and one without a "Source" entry, we don't find something here
        uri = self.cache._guess_third_party_changelogs_uri_by_source("apt")
        self.assertEqual(uri, None)
        # one with srcver == binver
        pkgname = "libgtk2.0-dev"
        uri = self.cache._guess_third_party_changelogs_uri_by_source(pkgname)
        pkg = self.cache[pkgname]
        self.assertTrue(pkg.candidate.version in uri)
        self.assertTrue("gtk+2.0" in uri)

    def test_changelog_not_supported(self):
        def monkey_patched_get_changelogs(name, what, ver, uri):
            with open("/dev/zero") as zero:
                raise HTTPError("url", "code", "msg", "hdrs", zero)

        pkgname = "gcc"
        # patch origin
        real_origin = self.cache.CHANGELOG_ORIGIN
        self.cache.CHANGELOG_ORIGIN = "xxx"
        # monkey patch to raise the right error
        self.cache._get_changelog_or_news = monkey_patched_get_changelogs
        # get changelog
        self.cache.get_changelog(pkgname)
        error = "This update does not come from a source that "
        error += "supports changelogs."
        # verify that we don't have the lines twice
        self.assertEqual(
            self.cache.all_changes[pkgname].split("\n")[-1], error
        )
        self.assertEqual(len(self.cache.all_changes[pkgname].split("\n")), 5)
        self.assertEqual(self.cache.all_changes[pkgname].count(error), 1)
        self.cache.CHANGELOG_ORIGIN = real_origin


if __name__ == "__main__":
    if len(sys.argv) > 1 and sys.argv[1] == "-v":
        logging.basicConfig(level=logging.DEBUG)
    unittest.main()



#!/usr/bin/python3
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-

import os
import shutil

import apt
import unittest

from UpdateManager.Core import UpdateList
from UpdateManager.Core.MyCache import MyCache

from gi.repository import Gio
from mock import patch, PropertyMock, MagicMock

CURDIR = os.path.dirname(os.path.abspath(__file__))


class PhasedTestCase(unittest.TestCase):
    def setUp(self):
        # mangle the arch
        real_arch = apt.apt_pkg.config.find("APT::Architecture")
        apt.apt_pkg.config.set("APT::Architecture", "amd64")
        self.addCleanup(
            lambda: apt.apt_pkg.config.set("APT::Architecture", real_arch)
        )

        self.aptroot = os.path.join(CURDIR, "aptroot-update-list-test")
        self.cache = MyCache(
            apt.progress.base.OpProgress(), rootdir=self.aptroot
        )
        self.cache.open()
        self.updates_list = UpdateList.UpdateList(parent=None)

    def tearDown(self):
        try:
            shutil.rmtree(os.path.join(self.aptroot, "var/cache/apt"))
        except OSError:
            pass

    def assertUpdatesListLen(self, nr):
        self.assertEqual(self.updates_list.num_updates, nr)

    def test_phased_percentage_not_included(self):
        """Test that updates above the threshold are not included"""
        with patch.object(self.updates_list.random, "randint") as mock_randint:
            # threshold is 10
            mock_randint.return_value = 11
            self.updates_list.update(self.cache)
            self.assertUpdatesListLen(1)

    def test_phased_percentage_included(self):
        """Test that updates below the threshold are included"""
        with patch.object(self.updates_list.random, "randint") as mock_randint:
            # threshold is 10
            mock_randint.return_value = 9
            self.updates_list.update(self.cache)
            self.assertUpdatesListLen(3)

    def test_second_phased_binary_not_included(self):
        """Test that there is no overlap between the source packages of the
        packages being ignored and installed"""
        with patch.object(self.updates_list.random, "randint") as mock_randint:
            mock_randint.return_value = 11
            self.updates_list.update(self.cache)
        ignored_srcs = set(
            [
                pkg.candidate.source_name
                for pkg in self.updates_list.ignored_phased_updates
            ]
        )
        group = self.updates_list.update_groups[0]
        install_srcs = set([x.pkg.candidate.source_name for x in group.items])
        self.assertEqual(ignored_srcs, set({"zsh"}))
        self.assertEqual(install_srcs, set({"apt"}))
        self.assertTrue(len(ignored_srcs & install_srcs) == 0)

    def test_phased_percentage_included_via_force(self):
        """Test that the "always" override config works"""
        # set config to force override
        apt.apt_pkg.config.set(
            self.updates_list.ALWAYS_INCLUDE_PHASED_UPDATES, "1"
        )
        self.addCleanup(
            lambda: apt.apt_pkg.config.set(
                self.updates_list.ALWAYS_INCLUDE_PHASED_UPDATES, "0"
            )
        )
        # ensure it's included even if it's above the threshold
        with patch.object(self.updates_list.random, "randint") as mock_randint:
            mock_randint.return_value = 100
            self.updates_list.update(self.cache)
            self.assertUpdatesListLen(3)

    def test_phased_percentage_excluded_via_force(self):
        """Test that the "never" override config works"""
        # set config to force override
        apt.apt_pkg.config.set(
            self.updates_list.NEVER_INCLUDE_PHASED_UPDATES, "1"
        )
        self.addCleanup(
            lambda: apt.apt_pkg.config.set(
                self.updates_list.NEVER_INCLUDE_PHASED_UPDATES, "0"
            )
        )
        # ensure it's excluded even if it's below the threshold
        with patch.object(self.updates_list.random, "randint") as mock_randint:
            mock_randint.return_value = 0
            self.updates_list.update(self.cache)
            self.assertUpdatesListLen(1)

    @patch("UpdateManager.Core.UpdateList.UpdateList._is_security_update")
    def test_phased_percentage_from_security(self, mock_security):
        """Test that updates from the security node go in"""
        # pretend all updates come from security for the sake of this test
        mock_security.return_value = True
        with patch.object(self.updates_list.random, "randint") as mock_randint:
            mock_randint.return_value = 100
            self.updates_list.update(self.cache)
            self.assertUpdatesListLen(3)


class GroupingTestCase(unittest.TestCase):
    # installed_files does not respect aptroot, so we have to patch it
    @patch("apt.package.Package.installed_files", new_callable=PropertyMock)
    @patch("gi.repository.Gio.DesktopAppInfo.new_from_filename")
    def setUp(self, mock_desktop, mock_installed):
        # mangle the arch
        real_arch = apt.apt_pkg.config.find("APT::Architecture")
        apt.apt_pkg.config.set("APT::Architecture", "amd64")
        self.addCleanup(
            lambda: apt.apt_pkg.config.set("APT::Architecture", real_arch)
        )

        def set_versioned_kernel_pkgs(real_versioned_kernel_pkgs):
            apt.apt_pkg.config.clear("APT::VersionedKernelPackages")
            for k in real_versioned_kernel_pkgs:
                apt.apt_pkg.config.set("APT::VersionedKernelPackages::", k)

        # override the kernel autoremove regex
        real_versioned_kernel_pkgs = apt.apt_pkg.config.value_list()
        set_versioned_kernel_pkgs(["linux-.*"])
        self.addCleanup(
            lambda: set_versioned_kernel_pkgs(real_versioned_kernel_pkgs)
        )

        self.aptroot = os.path.join(CURDIR, "aptroot-grouping-test")
        self.cache = MyCache(
            apt.progress.base.OpProgress(), rootdir=self.aptroot
        )
        self.cache.open()
        self.cache["linux-generic-to-remove"].mark_auto(True)
        self.cache["linux-headers-to-remove"].mark_auto(True)
        mock_installed.__get__ = self.fake_installed_files
        mock_desktop.side_effect = self.fake_desktop
        self.updates_list = UpdateList.UpdateList(parent=None, dist="lucid")
        self.updates_list.update(
            self.cache,
            duplicate_packages=["linux-generic-dup1", "linux-generic-dup2"],
        )

    def tearDown(self):
        try:
            shutil.rmtree(os.path.join(self.aptroot, "var/cache/apt"))
        except OSError:
            pass

    def fake_installed_files(self, mock_prop, pkg, pkg_class):
        if pkg.name == "installed-app":
            return ["/usr/share/applications/installed-app.desktop"]
        elif pkg.name == "installed-app-with-subitems":
            return ["/usr/share/applications/installed-app2.desktop"]
        else:
            return []

    def fake_desktop(self, path):
        # These can all be the same for our purposes
        app = MagicMock()
        app.get_filename.return_value = path
        app.get_display_name.return_value = "App " + os.path.basename(path)
        app.get_icon.return_value = Gio.ThemedIcon.new("package")
        return app

    def test_app(self):
        self.assertGreater(len(self.updates_list.update_groups), 0)
        group = self.updates_list.update_groups[0]
        self.assertIsInstance(group, UpdateList.UpdateApplicationGroup)
        self.assertIsNotNone(group.core_item)
        self.assertEqual(group.core_item.pkg.name, "installed-app")
        self.assertListEqual(
            [x.pkg.name for x in group.items], ["installed-app"]
        )

    def test_app_with_subitems(self):
        self.assertGreater(len(self.updates_list.update_groups), 1)
        group = self.updates_list.update_groups[1]
        self.assertIsInstance(group, UpdateList.UpdateApplicationGroup)
        self.assertIsNotNone(group.core_item)
        self.assertEqual(
            group.core_item.pkg.name, "installed-app-with-subitems"
        )
        self.assertListEqual(
            [x.pkg.name for x in group.items],
            ["installed-app-with-subitems", "installed-pkg-single-dep"],
        )

    def test_pkg(self):
        self.assertGreater(len(self.updates_list.update_groups), 2)
        group = self.updates_list.update_groups[2]
        self.assertIsInstance(group, UpdateList.UpdatePackageGroup)
        self.assertIsNotNone(group.core_item)
        self.assertEqual(group.core_item.pkg.name, "installed-pkg")
        self.assertListEqual(
            [x.pkg.name for x in group.items], ["installed-pkg"]
        )

    def test_negative_regex_match(self):
        self.assertGreater(len(self.updates_list.update_groups), 3)
        group = self.updates_list.update_groups[3]
        self.assertIsInstance(group, UpdateList.UpdatePackageGroup)
        self.assertIsNotNone(group.core_item)
        self.assertEqual(group.core_item.pkg.name, "linux-show-player")
        self.assertListEqual(
            [x.pkg.name for x in group.items], ["linux-show-player"]
        )

    def test_pkg_multiple_deps(self):
        self.assertGreater(len(self.updates_list.update_groups), 4)
        group = self.updates_list.update_groups[4]
        self.assertIsInstance(group, UpdateList.UpdatePackageGroup)
        self.assertIsNotNone(group.core_item)
        self.assertEqual(
            group.core_item.pkg.name, "installed-pkg-multiple-deps"
        )
        self.assertListEqual(
            [x.pkg.name for x in group.items], ["installed-pkg-multiple-deps"]
        )

    def test_security(self):
        self.assertEqual(len(self.updates_list.security_groups), 1)
        group = self.updates_list.security_groups[0]
        self.assertIsInstance(group, UpdateList.UpdateSystemGroup)
        self.assertIsNone(group.core_item)
        self.assertListEqual([x.pkg.name for x in group.items], ["base-pkg"])

    def test_base_grouping(self):
        self.assertEqual(len(self.updates_list.update_groups), 6)
        group = self.updates_list.update_groups[5]
        self.assertIsInstance(group, UpdateList.UpdateSystemGroup)
        self.assertIsNone(group.core_item)
        self.assertListEqual(
            [x.pkg.name for x in group.items],
            ["ubuntu-minimal", "linux-test-hwe"],
        )

    def test_autoremove_grouping(self):
        # test that packages staged for removal aren't grouped if they are
        # kernel autoremove packages (LP: #1912718)
        self.assertGreater(len(self.updates_list.kernel_autoremove_groups), 0)
        group = self.updates_list.kernel_autoremove_groups[0]
        self.assertIsInstance(group, UpdateList.UpdatePackageGroup)
        self.assertIsNotNone(group.core_item)
        self.assertListEqual(
            [x.pkg.name for x in group.items], ["linux-generic-to-remove"]
        )

        self.assertEqual(len(self.updates_list.kernel_autoremove_groups), 2)
        group = self.updates_list.kernel_autoremove_groups[1]
        self.assertIsInstance(group, UpdateList.UpdatePackageGroup)
        self.assertIsNotNone(group.core_item)
        self.assertListEqual(
            [x.pkg.name for x in group.items], ["linux-headers-to-remove"]
        )

    def test_duplicate_grouping(self):
        # test that packages staged for removal aren't grouped if they are
        # duplicate packages (LP: #1912718)
        self.assertGreater(len(self.updates_list.duplicate_groups), 0)
        group = self.updates_list.duplicate_groups[0]
        self.assertIsInstance(group, UpdateList.UpdatePackageGroup)
        self.assertIsNotNone(group.core_item)
        self.assertListEqual(
            [x.pkg.name for x in group.items], ["linux-generic-dup1"]
        )

        self.assertEqual(len(self.updates_list.duplicate_groups), 2)
        group = self.updates_list.duplicate_groups[1]
        self.assertIsInstance(group, UpdateList.UpdatePackageGroup)
        self.assertIsNotNone(group.core_item)
        self.assertListEqual(
            [x.pkg.name for x in group.items], ["linux-generic-dup2"]
        )


if __name__ == "__main__":
    unittest.main()



# helpviewer.py
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-

import os
import subprocess

# Hardcoded list of available help viewers
# FIXME: khelpcenter support would be nice
# KNOWN_VIEWERS = ["/usr/bin/yelp", "/usr/bin/khelpcenter"]
KNOWN_VIEWERS = ["/usr/bin/yelp"]


class HelpViewer:
    def __init__(self, docu):
        self.command = []
        self.docu = docu
        for viewer in KNOWN_VIEWERS:
            if os.path.exists(viewer):
                self.command = [viewer, "help:%s" % docu]
                break

    def check(self):
        """check if a viewer is available"""
        if self.command == []:
            return False
        else:
            return True

    def run(self):
        """open the documentation in the viewer"""
        # avoid running the help viewer as root
        if os.getuid() == 0 and "SUDO_USER" in os.environ:
            self.command = [
                "sudo",
                "-u",
                os.environ["SUDO_USER"],
            ] + self.command
        subprocess.Popen(self.command)



# ChangelogViewer.py
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
#  Copyright (c) 2006 Sebastian Heinlein
#                2007 Canonical
#
#  Author: Sebastian Heinlein <sebastian.heinlein@web.de>
#          Michael Vogt <michael.vogt@ubuntu.com>
#
#  This modul provides an inheritance of the Gtk.TextView that is
#  aware of http URLs and allows to open them in a browser.
#  It is based on the pygtk-demo "hypertext".
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA

import gi

gi.require_version("Gtk", "3.0")
from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import GObject
from gi.repository import Pango
from gettext import gettext as _

from DistUpgrade.ReleaseNotesViewer import open_url


class ChangelogViewer(Gtk.TextView):
    def __init__(self, changelog=None):
        """Init the ChangelogViewer as an Inheritance of the Gtk.TextView"""
        # init the parent
        GObject.GObject.__init__(self)
        # global hovering over link state
        self.hovering = False
        self.first = True
        # setup the buffer and signals
        self.set_property("editable", False)
        self.set_cursor_visible(False)
        # set some margin
        self.set_right_margin(4)
        self.set_left_margin(4)
        self.set_pixels_above_lines(4)
        # fill the area
        self.set_vexpand(True)
        self.buffer = Gtk.TextBuffer()
        self.set_buffer(self.buffer)
        self.connect("button-press-event", self.button_press_event)
        self.connect("motion-notify-event", self.motion_notify_event)
        self.connect("visibility-notify-event", self.visibility_notify_event)
        # self.buffer.connect("changed", self.search_links)
        self.buffer.connect_after("insert-text", self.on_insert_text)
        # search for links in the changelog and make them clickable
        if changelog is not None:
            self.buffer.set_text(changelog)

    def create_context_menu(self, url):
        """Create the context menu to be displayed when links are right
        clicked"""
        self.menu = Gtk.Menu()

        # create menu items
        item_grey_link = Gtk.MenuItem()
        item_grey_link.set_label(url)
        item_grey_link.connect(
            "activate", self.handle_context_menu, "open", url
        )
        item_seperator = Gtk.MenuItem()
        item_open_link = Gtk.MenuItem()
        item_open_link.set_label(_("Open Link in Browser"))
        item_open_link.connect(
            "activate", self.handle_context_menu, "open", url
        )
        item_copy_link = Gtk.MenuItem()
        item_copy_link.set_label(_("Copy Link to Clipboard"))
        item_copy_link.connect(
            "activate", self.handle_context_menu, "copy", url
        )

        # add menu items
        self.menu.add(item_grey_link)
        self.menu.add(item_seperator)
        self.menu.add(item_open_link)
        self.menu.add(item_copy_link)
        self.menu.show_all()

    def handle_context_menu(self, menuitem, action, url):
        """Handle activate event for the links' context menu"""
        if action == "open":
            open_url(url)
        if action == "copy":
            # the following two lines used to be enough - then gtk3/pygi
            # came along ...
            # cb = Gtk.Clipboard()
            # cb.set_text(url)
            display = Gdk.Display.get_default()
            selection = Gdk.Atom.intern("CLIPBOARD", False)
            cb = Gtk.Clipboard.get_for_display(display, selection)
            cb.set_text(url, -1)
            cb.store()

    def tag_link(self, start, end, url):
        """Apply the tag that marks links to the specified buffer selection"""
        tags = start.get_tags()
        for tag in tags:
            url = getattr(tag, "url", None)
            if url != "":
                return
        tag = self.buffer.create_tag(
            None, foreground="blue", underline=Pango.Underline.SINGLE
        )
        tag.url = url
        self.buffer.apply_tag(tag, start, end)

    def on_insert_text(self, buffer, iter_end, content, *args):
        """Search for http URLs in newly inserted text
        and tag them accordingly"""

        # some convenient urls
        MALONE = "https://launchpad.net/bugs/"
        DEBIAN = "http://bugs.debian.org/"
        CVE = "http://cve.mitre.org/cgi-bin/cvename.cgi?name=CVE-"
        # some convinient end-markers
        ws = [" ", "\t", "\n"]
        brak = [")", "]", ">"]
        punct = [",", "!", ":"]
        dot = ["."] + punct
        dot_cr = [".\n"]

        # search items are start-str, list-of-end-strs, url-prefix
        # a lot of this search is "TEH SUCK"(tm) because of limitations
        # in iter.forward_search()
        # - i.e. no insensitive searching, no regexp
        search_items = [
            ("http://", ws + brak + punct + dot_cr, "http://"),
            ("LP#", ws + brak + dot, MALONE),
            ("lp#", ws + brak + dot, MALONE),
            ("LP: #", ws + brak + dot, MALONE),
            ("lp: #", ws + brak + dot, MALONE),
            ("LP:#", ws + brak + dot, MALONE),
            ("lp:#", ws + brak + dot, MALONE),
            ("Malone: #", ws + brak + dot, MALONE),
            ("Malone:#", ws + brak + dot, MALONE),
            ("Ubuntu: #", ws + brak + dot, MALONE),
            ("Ubuntu:#", ws + brak + dot, MALONE),
            ("Closes: #", ws + brak + dot, DEBIAN),
            ("Closes:#", ws + brak + dot, DEBIAN),
            ("closes:#", ws + brak + dot, DEBIAN),
            ("closes: #", ws + brak + dot, DEBIAN),
            ("CVE-", ws + brak + dot, CVE),
        ]

        # search for the next match in the buffer
        for (start_str, end_list, url_prefix) in search_items:
            # init
            iter = buffer.get_iter_at_offset(
                iter_end.get_offset() - len(content)
            )
            while True:
                ret = iter.forward_search(
                    start_str, Gtk.TextSearchFlags.VISIBLE_ONLY, iter_end
                )
                # if we reach the end break the loop
                if not ret:
                    break
                # get the position of the protocol prefix
                (match_start, match_end) = ret
                match_suffix = match_end.copy()
                match_tmp = match_end.copy()
                while True:
                    # extend the selection to the complete search item
                    if match_tmp.forward_char():
                        text = match_end.get_text(match_tmp)
                        if text in end_list:
                            break
                        # move one char futher to get two char
                        # end-markers (and back later) LP: #396393
                        match_tmp.forward_char()
                        text = match_end.get_text(match_tmp)
                        if text in end_list:
                            break
                        match_tmp.backward_char()
                    else:
                        break
                    match_end = match_tmp.copy()

                # call the tagging method for the complete URL
                url = url_prefix + match_suffix.get_text(match_end)

                self.tag_link(match_start, match_end, url)
                # set the starting point for the next search
                iter = match_end

    def button_press_event(self, text_view, event):
        """callback for mouse click events"""
        # we only react on left or right mouse clicks
        if event.button != 1 and event.button != 3:
            return False

        # try to get a selection
        try:
            (start, end) = self.buffer.get_selection_bounds()
        except ValueError:
            pass
        else:
            if start.get_offset() != end.get_offset():
                return False

        # get the iter at the mouse position
        (x, y) = self.window_to_buffer_coords(
            Gtk.TextWindowType.WIDGET, int(event.x), int(event.y)
        )
        # call open_url or menu.popup if an URL is assigned to the iter
        try:
            iter = self.get_iter_at_location(x, y)
            tags = iter.get_tags()
        except AttributeError:
            # GTK > 3.20 added a return type to this function
            (over_text, iter) = self.get_iter_at_location(x, y)
            tags = iter.get_tags()

        for tag in tags:
            if hasattr(tag, "url"):
                if event.button == 1:
                    open_url(tag.url)
                    break
                if event.button == 3:
                    self.create_context_menu(tag.url)
                    self.menu.popup(
                        None, None, None, None, event.button, event.time
                    )
                    return True

    def motion_notify_event(self, text_view, event):
        """callback for the mouse movement event, that calls the
        check_hovering method with the mouse postition coordiantes"""
        x, y = text_view.window_to_buffer_coords(
            Gtk.TextWindowType.WIDGET, int(event.x), int(event.y)
        )
        self.check_hovering(x, y)
        self.get_window(Gtk.TextWindowType.TEXT).get_pointer()
        return False

    def visibility_notify_event(self, text_view, event):
        """callback if the widgets gets visible (e.g. moves to the foreground)
        that calls the check_hovering method with the mouse position
        coordinates"""
        window = text_view.get_window(Gtk.TextWindowType.TEXT)
        (screen, wx, wy, mod) = window.get_pointer()
        (bx, by) = text_view.window_to_buffer_coords(
            Gtk.TextWindowType.WIDGET, wx, wy
        )
        self.check_hovering(bx, by)
        return False

    def check_hovering(self, x, y):
        """Check if the mouse is above a tagged link and if yes show
        a hand cursor"""
        _hovering = False
        # get the iter at the mouse position
        try:
            iter = self.get_iter_at_location(x, y)
            tags = iter.get_tags()
        except AttributeError:
            # GTK > 3.20 added a return type to this function
            (over_text, iter) = self.get_iter_at_location(x, y)
            tags = iter.get_tags()

        # set _hovering if the iter has the tag "url"
        for tag in tags:
            if hasattr(tag, "url"):
                _hovering = True
                break

        # change the global hovering state
        if _hovering != self.hovering or self.first:
            self.first = False
            self.hovering = _hovering
            # Set the appropriate cursur icon
            if self.hovering:
                self.get_window(Gtk.TextWindowType.TEXT).set_cursor(
                    Gdk.Cursor.new(Gdk.CursorType.HAND2)
                )
            else:
                self.get_window(Gtk.TextWindowType.TEXT).set_cursor(
                    Gdk.Cursor.new(Gdk.CursorType.LEFT_PTR)
                )


if __name__ == "__main__":
    w = Gtk.Window()
    cv = ChangelogViewer()
    changes = cv.get_buffer()
    changes.create_tag("versiontag", weight=Pango.Weight.BOLD)
    changes.set_text(
        """

Version 6-14-0ubuntu1.9.04:

  * New upstream version. LP: #382918.
    Release notes at http://java.sun.com/javase/6/webnotes/ReleaseNotes.html.

"""
    )

    w.add(cv)
    w.show_all()
    Gtk.main()



# Copyright (C) 2009  Canonical, Ltd.
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, version 3 of the License.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

"""Remove lilo if grub is also installed."""

__metaclass__ = type
__all__ = ["RemoveLiloPlugin"]


import os
import logging

from janitor.plugincore.i18n import setup_gettext
from janitor.plugincore.core.package_cruft import PackageCruft
from janitor.plugincore.plugin import Plugin

_ = setup_gettext()


class RemoveLiloPlugin(Plugin):
    """Plugin to remove lilo if grub is also installed."""

    def __init__(self):
        self.condition = ["jauntyPostDistUpgradeCache"]

    def get_description(self):
        return _(
            "Remove lilo since grub is also installed."
            "(See bug #314004 for details.)"
        )

    def get_cruft(self):
        if "lilo" in self.app.apt_cache and "grub" in self.app.apt_cache:
            lilo = self.app.apt_cache["lilo"]
            grub = self.app.apt_cache["grub"]
            if lilo.is_installed and grub.is_installed:
                if not os.path.exists("/etc/lilo.conf"):
                    yield PackageCruft(lilo, self.description)
                else:
                    logging.warning(
                        "lilo and grub installed, but lilo.conf exists"
                    )



#!/usr/bin/env python
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-

"""Integration of package managers into UpdateManager"""
# (c) 2005-2009 Canonical, GPL

import gi

gi.require_version("Gtk", "3.0")
gi.require_version("Snapd", "2")
from gi.repository import GLib, Gtk, Snapd

from apt import Cache
import json
import logging
import os
import re
import subprocess
from gettext import gettext as _
from threading import Thread

from UpdateManager.Core.MyCache import MyCache
from UpdateManager.Core.utils import inhibit_sleep, get_dist_version
from UpdateManager.Dialogs import Dialog


class InstallBackend(Dialog):
    ACTION_UPDATE = 0
    ACTION_PRE_INSTALL = 1
    ACTION_INSTALL = 2

    def __init__(self, window_main, action):
        Dialog.__init__(self, window_main)
        self.action = action
        self.sleep_cookie = None

    def start(self):
        os.environ["APT_LISTCHANGES_FRONTEND"] = "none"

        # Do not suspend during the update process
        self.sleep_cookie = inhibit_sleep()

        if self.action == self.ACTION_PRE_INSTALL:
            unfresh_cache = self.window_main.cache
            fresh_cache = Cache(rootdir=self.window_main.cache.rootdir)
            # Install OEM packages, update, then do ACTION_INSTALL
            pkgs_install_oem = []
            pkgs_upgrade_oem = []
            for pkg in self.window_main.oem_metapackages:
                unfresh_pkg = unfresh_cache[pkg]
                fresh_pkg = fresh_cache[pkg]

                if unfresh_pkg.marked_install and not fresh_pkg.is_installed:
                    pkgs_install_oem.append(pkg)
                elif unfresh_pkg.marked_upgrade and fresh_pkg.is_upgradable:
                    pkgs_upgrade_oem.append(pkg)
            self.commit_oem(pkgs_install_oem, pkgs_upgrade_oem)
        elif self.action == self.ACTION_INSTALL:
            # Get the packages which should be installed and update
            pkgs_install = []
            pkgs_upgrade = []
            pkgs_remove = []
            # Get a fresh cache in case update-manager's is outdated to
            # skip operations that already took place
            fresh_cache = Cache(rootdir=self.window_main.cache.rootdir)
            for pkg in self.window_main.cache:
                try:
                    if (
                        pkg.marked_install
                        and not fresh_cache[pkg.name].is_installed
                    ):
                        pkgname = pkg.name
                        if pkg.is_auto_installed:
                            pkgname += "#auto"
                        pkgs_install.append(pkgname)
                    elif (
                        pkg.marked_upgrade
                        and fresh_cache[pkg.name].is_upgradable
                    ):
                        pkgs_upgrade.append(pkg.name)
                    elif (
                        pkg.marked_delete
                        and fresh_cache[pkg.name].is_installed
                    ):
                        pkgs_remove.append(pkg.name)
                except KeyError:
                    # pkg missing from fresh_cache can't be modified
                    pass
            self.commit(pkgs_install, pkgs_upgrade, pkgs_remove)
        else:
            self.update()

    def update(self):
        """Run a update to refresh the package list"""
        raise NotImplementedError

    def commit_oem(self, pkgs_install_oem, pkgs_upgrade_oem):
        """Install these OEM packages"""
        self._action_done(
            self.ACTION_PRE_INSTALL,
            authorized=True,
            success=True,
            error_string=None,
            error_desc=None,
            trans_failed=None,
        )

    def commit(self, pkgs_install, pkgs_upgrade, pkgs_remove):
        """Commit the cache changes"""
        raise NotImplementedError

    def get_snap_seeds(self):
        seeded_snaps = {}
        unseeded_snaps = {}

        curr_channel = "stable/ubuntu-" + get_dist_version()
        cache = self.window_main.cache

        try:
            d2s_file = open(
                "/usr/share/ubuntu-release-upgrader/deb2snap.json", "r"
            )
            d2s = json.load(d2s_file)
            d2s_file.close()

            for snap in d2s["seeded"]:
                seed = d2s["seeded"][snap]
                deb = seed.get("deb", None)
                to_channel = seed.get("to_channel", curr_channel)
                metapkg = seed.get("metapkg", None)
                if metapkg not in cache:
                    continue
                if metapkg and cache[metapkg].is_installed is False:
                    continue
                seeded_snaps[snap] = (deb, to_channel)

            for snap in d2s["unseeded"]:
                unseed = d2s["unseeded"][snap]
                from_channel = unseed.get("from_channel", curr_channel)
                metapkg = seed.get("metapkg", None)
                if metapkg not in cache:
                    continue
                if metapkg and cache[metapkg].is_installed is False:
                    continue
                unseeded_snaps[snap] = from_channel
        except Exception as e:
            logging.debug("error reading deb2snap.json file (%s)" % e)

        return seeded_snaps, unseeded_snaps

    def get_deb2snap_dups(self):
        # update and grab the latest cache
        try:
            if self.window_main.cache is None:
                self.window_main.cache = MyCache(None)
            else:
                self.window_main.cache.open(None)
                self.window_main.cache._initDepCache()
            cache = self.window_main.cache
        except Exception as e:
            # just return an empty array for now, it's perfectly safe to
            # postpone this duplicates check to a later update.
            logging.debug("error reading cache (%s)" % e)
            return []

        duplicates = []
        seeded_snaps, _ = self.get_snap_seeds()

        for snap, (deb, _) in seeded_snaps.items():
            # if the deb is installed and was not manually installed,
            # replace it
            if deb in cache and cache[deb].is_installed:
                deb_is_auto = True
                cache[deb].mark_delete()

                for pkg in cache.get_changes():
                    if (
                        pkg.is_installed
                        and pkg.marked_delete
                        and not pkg.is_auto_installed
                    ):
                        deb_is_auto = False
                        break

                cache.clear()

                if deb_is_auto:
                    duplicates.append(deb)

        return duplicates

    def get_snap_transitions(self):
        # populate snap_list with deb2snap transitions
        snap_list = {}
        seeded_snaps, unseeded_snaps = self.get_snap_seeds()

        for snap, (deb, to_channel) in seeded_snaps.items():
            snap_object = {}
            # check if the snap is already installed
            snap_info = subprocess.Popen(
                ["snap", "info", snap],
                universal_newlines=True,
                stdout=subprocess.PIPE,
            ).communicate()
            if re.search("^installed: ", snap_info[0], re.MULTILINE):
                logging.debug("Snap %s is installed" % snap)
                continue
            elif deb in self.window_main.duplicate_packages:
                # install the snap if the deb was just marked delete
                snap_object["command"] = "install"
                snap_object["channel"] = to_channel
                snap_list[snap] = snap_object

        for snap, (from_channel) in unseeded_snaps.items():
            snap_object = {}
            # check if the snap is already installed
            snap_info = subprocess.Popen(
                ["snap", "info", snap],
                universal_newlines=True,
                stdout=subprocess.PIPE,
            ).communicate()
            if re.search("^installed: ", snap_info[0], re.MULTILINE):
                logging.debug("Snap %s is installed" % snap)
                # its not tracking the release channel so don't remove
                re_channel = "stable/ubuntu-[0-9][0-9].[0-9][0-9]"
                if not re.search(
                    r"^tracking:.*%s" % re_channel, snap_info[0], re.MULTILINE
                ):
                    logging.debug(
                        "Snap %s is not tracking the release channel" % snap
                    )
                    continue

                snap_object["command"] = "remove"

                # check if this snap is being used by any other snaps
                conns = subprocess.Popen(
                    ["snap", "connections", snap],
                    universal_newlines=True,
                    stdout=subprocess.PIPE,
                ).communicate()

                for conn in conns[0].split("\n"):
                    conn_cols = conn.split()
                    if len(conn_cols) != 4:
                        continue
                    plug = conn_cols[1]
                    slot = conn_cols[2]

                    if slot.startswith(snap + ":"):
                        plug_snap = plug.split(":")[0]
                        if (
                            plug_snap != "-"
                            and plug_snap not in unseeded_snaps
                        ):
                            logging.debug(
                                "Snap %s is being used by %s. "
                                "Switching it to stable track"
                                % (snap, plug_snap)
                            )
                            snap_object["command"] = "refresh"
                            snap_object["channel"] = "stable"
                            break

                snap_list[snap] = snap_object

        return snap_list

    def update_snap_cb(self, client, change, _, user_data):
        index, count, progress_bar = user_data
        if not progress_bar:
            return

        # determine how much of this change has been done
        task_total = 0
        task_done = 0
        for task in change.get_tasks():
            task_total += task.get_progress_total()
            task_done += task.get_progress_done()

        task_fraction = task_done / task_total

        # determine how much total progress has been made
        total_fraction = (task_fraction / count) + (index / count)

        # change.get_tasks() can increase between callbacks so we must
        # avoid jumping backward in progress here
        if total_fraction > progress_bar.get_fraction():
            GLib.idle_add(progress_bar.set_fraction, total_fraction)

    def update_snaps(self):
        # update status and progress bar
        def update_status(status):
            GLib.idle_add(self.label_details.set_label, status)

        def update_progress(progress_bar):
            progress_bar.pulse()
            return True

        update_status(_("Updating snaps"))

        progress_bar = None
        progress_timer = None

        progress_bars = self.progressbar_slot.get_children()
        if progress_bars and isinstance(progress_bars[0], Gtk.ProgressBar):
            progress_bar = progress_bars[0]
            progress_timer = GLib.timeout_add(
                100, update_progress, progress_bar
            )

        # populate snap_list with deb2snap transitions
        snap_list = self.get_snap_transitions()

        if progress_timer:
            GLib.source_remove(progress_timer)
            progress_bar.set_fraction(0)

        # (un)install (un)seeded snap(s)
        try:
            client = Snapd.Client()
            client.connect_sync()
            index = 0
            count = len(snap_list)
            for snap, snap_object in snap_list.items():
                command = snap_object["command"]
                if command == "refresh":
                    update_status(_("Refreshing %s snap" % snap))
                    client.refresh_sync(
                        snap,
                        snap_object["channel"],
                        self.update_snap_cb,
                        progress_callback_data=(index, count, progress_bar),
                    )
                elif command == "remove":
                    update_status(_("Removing %s snap" % snap))
                    client.remove_sync(
                        snap,
                        self.update_snap_cb,
                        progress_callback_data=(index, count, progress_bar),
                    )
                else:
                    update_status(_("Installing %s snap" % snap))
                    client.install_sync(
                        snap,
                        snap_object["channel"],
                        self.update_snap_cb,
                        progress_callback_data=(index, count, progress_bar),
                    )
                index += 1
        except GLib.Error as e:
            logging.debug("error updating snaps (%s)" % e)
            GLib.idle_add(
                self.window_main.start_error,
                False,
                _("Upgrade only partially completed."),
                _(
                    "An error occurred while updating snaps. "
                    "Please check your network connection."
                ),
            )
            return

        # continue with the rest of the updates
        GLib.idle_add(self.window_main.start_available)

    def _action_done(
        self,
        action,
        authorized,
        success,
        error_string,
        error_desc,
        trans_failed=False,
    ):

        # If the progress dialog should be closed automatically afterwards
        # settings = Gio.Settings.new("com.ubuntu.update-manager")
        # close_after_install = settings.get_boolean(
        #    "autoclose-install-window")
        # FIXME: confirm with mpt whether this should still be a setting
        # close_after_install = False

        if action == self.ACTION_PRE_INSTALL and success:
            # Now do the regular updates
            self.action = self.ACTION_INSTALL
            self.start()
        elif action == self.ACTION_INSTALL:
            if (
                success
                and os.path.exists("/usr/bin/snap")
                and hasattr(self, "pane_update_progress")
            ):
                Thread(target=self.update_snaps).start()
            elif success:
                self.window_main.start_available()
            elif error_string:
                self.window_main.start_error(
                    trans_failed, error_string, error_desc
                )
            else:
                # exit gracefuly, we can't just exit as this will trigger
                # a crash if system.exit() is called in a exception handler
                GLib.timeout_add(1, self.window_main.exit)
        else:
            if error_string:
                self.window_main.start_error(True, error_string, error_desc)
            elif (
                success
                and os.path.exists("/usr/bin/snap")
                and hasattr(self, "pane_update_progress")
            ):
                self.window_main.duplicate_packages = self.get_deb2snap_dups()
                self.window_main.start_available()
            else:
                is_cancelled_update = not success
                self.window_main.start_available(is_cancelled_update)


# try aptdaemon
if (
    os.path.exists("/usr/sbin/aptd")
    and "UPDATE_MANAGER_FORCE_BACKEND_SYNAPTIC" not in os.environ
):
    # check if the gtkwidgets are installed as well
    try:
        from .InstallBackendAptdaemon import InstallBackendAptdaemon
    except ImportError:
        logging.exception("importing aptdaemon")
# try synaptic
if (
    os.path.exists("/usr/sbin/synaptic")
    and "UPDATE_MANAGER_FORCE_BACKEND_APTDAEMON" not in os.environ
):
    try:
        from .InstallBackendSynaptic import InstallBackendSynaptic
    except ImportError:
        logging.exception("importing synaptic")


def get_backend(*args, **kwargs):
    """Select and return a package manager backend."""
    # try aptdaemon
    if (
        os.path.exists("/usr/sbin/aptd")
        and "UPDATE_MANAGER_FORCE_BACKEND_SYNAPTIC" not in os.environ
    ):
        # check if the gtkwidgets are installed as well
        try:
            return InstallBackendAptdaemon(*args, **kwargs)
        except NameError:
            logging.exception("using aptdaemon failed")
    # try synaptic
    if (
        os.path.exists("/usr/sbin/synaptic")
        and "UPDATE_MANAGER_FORCE_BACKEND_APTDAEMON" not in os.environ
    ):
        try:
            return InstallBackendSynaptic(*args, **kwargs)
        except NameError:
            pass
    # nothing found, raise
    raise Exception(
        "No working backend found, please try installing "
        "aptdaemon or synaptic"
    )



# UnitySupport.py
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
#  Copyright (c) 2011 Canonical
#
#  Author: Michael Vogt <mvo@ubuntu.com>
#          Bilal Akhtar <bilalakhtar@ubuntu.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA

# This file adds Unity integration, namely:
# In the launcher icon in the dock, there are proxy:
#   - progess bars to Update Manager's progress bars.
#   - menu entries to some Update Manager's actions.
# and the launcher will also reflect the urgency hint.

import logging
from gettext import gettext as _

HAVE_UNITY_SUPPORT = False
try:
    import gi

    gi.require_version("Dbusmenu", "0.4")
    gi.require_version("Unity", "7.0")
    from gi.repository import Dbusmenu, Unity

    HAVE_UNITY_SUPPORT = True
except (ValueError, ImportError, TypeError) as e:
    logging.warning("can not import unity GI %s" % e)


class IUnitySupport(object):
    """interface for unity support"""

    def __init__(self, parent=None):
        pass

    def set_urgency(self, urgent):
        pass

    def set_install_menuitem_visible(self, visible):
        pass

    def set_progress(self, progress):
        pass


class UnitySupportImpl(IUnitySupport):
    """implementation of unity support (if unity is available)"""

    def __init__(self, parent=None):
        # create launcher and quicklist
        um_launcher_entry = Unity.LauncherEntry.get_for_desktop_id(
            "update-manager.desktop"
        )
        self._unity = um_launcher_entry
        if parent:
            self._add_quicklist(parent)

    def _add_quicklist(self, parent):
        quicklist = Dbusmenu.Menuitem.new()
        # install
        self.install_dbusmenuitem = Dbusmenu.Menuitem.new()
        self.install_dbusmenuitem.property_set(
            Dbusmenu.MENUITEM_PROP_LABEL, _("Install All Available Updates")
        )
        self.install_dbusmenuitem.property_set_bool(
            Dbusmenu.MENUITEM_PROP_VISIBLE, True
        )
        self.install_dbusmenuitem.connect(
            "item-activated", parent.install_all_updates, None
        )
        quicklist.child_append(self.install_dbusmenuitem)
        # add it
        self._unity.set_property("quicklist", quicklist)

    def set_progress(self, progress):
        """set the progress [0,100]"""
        self._unity.set_property("progress", progress / 100.0)
        # hide progress when out of bounds
        if progress < 0 or progress > 100:
            self._unity.set_property("progress_visible", False)
        else:
            self._unity.set_property("progress_visible", True)

    def set_urgency(self, urgent):
        self._unity.set_property("urgent", urgent)

    def set_install_menuitem_visible(self, visible):
        self.install_dbusmenuitem.property_set_bool(
            Dbusmenu.MENUITEM_PROP_VISIBLE, visible
        )


# check what to export to the clients
if HAVE_UNITY_SUPPORT:
    UnitySupport = UnitySupportImpl
else:
    # we just provide the empty interface
    UnitySupport = IUnitySupport



# Copyright (C) 2008-2012  Canonical, Ltd.
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, version 3 of the License.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

__metaclass__ = type
__all__ = ["Application", "MockAptPackage", "setup_plugins"]


import os
import shutil
import tempfile
import pkg_resources


def setup_plugins(*plugin_filenames):
    plugin_dir = tempfile.mkdtemp()
    for filename in plugin_filenames:
        src = pkg_resources.resource_filename(
            "janitor.plugincore.tests.data", filename
        )
        dst = os.path.join(plugin_dir, filename)
        shutil.copyfile(src, dst)
    return (plugin_dir, lambda: shutil.rmtree(plugin_dir))


class Application:
    def __init__(self):
        self.notifications = []
        self.commit_called = False
        self.refresh_called = False
        self.apt_cache = self

    def commit(self, foo, bar):
        self.commit_called = True

    def refresh_apt_cache(self):
        self.refresh_called = True


class MockAptPackage:
    def __init__(self):
        self.name = "name"
        self.summary = "summary"
        self.installedSize = 12765
        self.installed = False
        self.deleted = False

    def markInstall(self):
        self.installed = True

    def markDelete(self):
        self.deleted = True



#!/usr/bin/python3


import apt
import builtins
import unittest
import os

from unittest import TestCase
from mock import patch

import hwe_support_status

builtins  # pyflakes


def make_mock_pkg(name, ver_str="1.0"):
    arch = apt.apt_pkg.config.find("APT::Architecture")
    return hwe_support_status.Package(name, ver_str, arch)


class HweSupportStatusTestCase(TestCase):

    INSTALLED_UNSUPPORTED_HWE_KERNEL_PKG_NAMES = (
        "linux-generic-lts-utopic",
        "linux-image-3.16.0-20-generic",
    )
    INSTALLED_UNSUPPORTED_HWE_XORG_PKG_NAMES = (
        "xserver-xorg-core-lts-utopic",
    )
    INSTALLED_SUPPORTED_HWE_PKG_NAMES = ("xserver-xorg-core-lts-trusty",)
    INSTALLED_OTHER_PKG_NAMES = ("2vcard",)

    INSTALLED_UNSUPPORTED_HWE_PKG_NAMES = (
        INSTALLED_UNSUPPORTED_HWE_KERNEL_PKG_NAMES
        + INSTALLED_UNSUPPORTED_HWE_XORG_PKG_NAMES
    )

    def setUp(self):
        self.cache = []
        INSTALL = (
            self.INSTALLED_UNSUPPORTED_HWE_PKG_NAMES
            + self.INSTALLED_SUPPORTED_HWE_PKG_NAMES
            + self.INSTALLED_OTHER_PKG_NAMES
        )
        for name in INSTALL:
            self.cache.append(make_mock_pkg(name))

    def test_find_unsupported_hwe_packages(self):
        unsupported, supported = hwe_support_status.find_hwe_packages(
            self.cache
        )
        self.assertEqual(
            set([pkg.name for pkg in unsupported]),
            set(self.INSTALLED_UNSUPPORTED_HWE_PKG_NAMES),
        )

    def test_is_unsupported_hwe_kernel_running(self):
        running_kver = os.uname()[2]
        mock_installed_kernel_pkg = make_mock_pkg(
            self.INSTALLED_UNSUPPORTED_HWE_KERNEL_PKG_NAMES[0],
            ver_str=running_kver,
        )
        unsupported = [mock_installed_kernel_pkg]
        self.assertTrue(
            hwe_support_status.is_unsupported_hwe_kernel_running(unsupported)
        )

    def test_is_unsupported_hwe_xorg_running(self):
        mock_installed_xorg_pkg = make_mock_pkg(
            self.INSTALLED_UNSUPPORTED_HWE_XORG_PKG_NAMES[0]
        )
        unsupported = [mock_installed_xorg_pkg]
        self.assertTrue(
            hwe_support_status.is_unsupported_xstack_running(unsupported)
        )

    def test_advice_about_hwe_status_is_running(self):
        with patch("hwe_support_status.is_unsupported_hwe_running") as m:
            with patch("builtins.print") as mock_print:
                m.return_value = True
                mock_unsupported_hwe_pkgs = []
                mock_unsupported_hwe_pkgs.append(
                    make_mock_pkg(
                        self.INSTALLED_UNSUPPORTED_HWE_XORG_PKG_NAMES[0]
                    )
                )
                from HweSupportStatus.consts import HWE_EOL_DATE
                from datetime import timedelta

                today = HWE_EOL_DATE - timedelta(14)
                hwe_support_status.advice_about_hwe_status(
                    mock_unsupported_hwe_pkgs,
                    [],
                    [],
                    False,
                    today,
                    verbose=True,
                )
                text = mock_print.call_args[0][0]
                self.assertIn(
                    "Your current Hardware Enablement Stack (HWE) "
                    "is going out of support",
                    text,
                )

    def test_advice_about_hwe_status_installed_only(self):
        with patch("hwe_support_status.is_unsupported_hwe_running") as m:
            with patch("builtins.print") as mock_print:
                m.return_value = False
                mock_unsupported_hwe_pkgs = []
                mock_unsupported_hwe_pkgs.append(
                    make_mock_pkg(
                        self.INSTALLED_UNSUPPORTED_HWE_XORG_PKG_NAMES[0]
                    )
                )
                from HweSupportStatus.consts import HWE_EOL_DATE
                from datetime import timedelta

                today = HWE_EOL_DATE + timedelta(14)
                hwe_support_status.advice_about_hwe_status(
                    mock_unsupported_hwe_pkgs,
                    [],
                    [],
                    False,
                    today,
                    verbose=True,
                )
                text = mock_print.call_args[0][0]
                self.assertIn(
                    "You have packages from the Hardware "
                    "Enablement Stack (HWE) installed that",
                    text,
                )

    def test_advice_about_hwe_status_no_hwe_stack(self):
        with patch("hwe_support_status.is_unsupported_hwe_running") as m:
            with patch("builtins.print") as mock_print:
                m.return_value = False
                mock_unsupported_hwe_pkgs = []
                from HweSupportStatus.consts import LTS_EOL_DATE
                from datetime import timedelta

                today = LTS_EOL_DATE - timedelta(14)
                hwe_support_status.advice_about_hwe_status(
                    mock_unsupported_hwe_pkgs,
                    [],
                    [],
                    False,
                    today,
                    verbose=True,
                )
                text = mock_print.call_args[0][0]
                print("43242343243", mock_print.call_count)
                self.assertIn(
                    "Your system is supported until May 2031", text
                )

    def test_advice_about_hwe_status_supported_hwe_stack(self):
        with patch("hwe_support_status.is_unsupported_hwe_running") as m:
            with patch("builtins.print") as mock_print:
                m.return_value = False
                mock_supported_hwe_pkgs = []
                mock_supported_hwe_pkgs.append(
                    make_mock_pkg(self.INSTALLED_SUPPORTED_HWE_PKG_NAMES[0])
                )
                from HweSupportStatus.consts import LTS_EOL_DATE
                from datetime import timedelta

                today = LTS_EOL_DATE - timedelta(14)
                hwe_support_status.advice_about_hwe_status(
                    [], mock_supported_hwe_pkgs, [], False, today, verbose=True
                )
                text = mock_print.call_args[0][0]
                self.assertIn(
                    "Your Hardware Enablement Stack (HWE) is supported "
                    "until May 2031",
                    text,
                )


if __name__ == "__main__":
    unittest.main()



#  Copyright (c) 2004-2007 Canonical
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
#  Author: Michael Vogt <michael.vogt@ubuntu.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA

from gi.repository import GLib
from gi.repository import GObject
from .Core.MetaRelease import MetaReleaseCore


class MetaRelease(MetaReleaseCore, GObject.GObject):

    __gsignals__ = {
        "new_dist_available": (
            GObject.SignalFlags.RUN_LAST,
            None,
            (GObject.TYPE_PYOBJECT,),
        ),
        "dist_no_longer_supported": (GObject.SignalFlags.RUN_LAST, None, ()),
        "done_downloading": (GObject.SignalFlags.RUN_LAST, None, ()),
    }

    def __init__(
        self, useDevelopmentRelease=False, useProposed=False, debug=False
    ):
        GObject.GObject.__init__(self)
        MetaReleaseCore.__init__(
            self, useDevelopmentRelease, useProposed, debug
        )
        # in the gtk space to test if the download already finished
        # this is needed because gtk is not thread-safe
        GLib.timeout_add_seconds(1, self.check)

    def check(self):
        # check if we have a metarelease_information file
        if self.no_longer_supported is not None:
            self.emit("dist_no_longer_supported")
        if self.new_dist is not None:
            self.emit("new_dist_available", self.new_dist)
        if self.downloading:
            return True
        else:
            self.emit("done_downloading")
            return False



# Copyright (C) 2009-2012  Canonical, Ltd.
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, version 3 of the License.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

__metaclass__ = type
__all__ = ["DpkgStatusCruft", "DpkgStatusPlugin"]


import logging
import subprocess

from apt_pkg import TagFile

from janitor.plugincore.cruft import Cruft
from janitor.plugincore.i18n import setup_gettext
from janitor.plugincore.plugin import Plugin

_ = setup_gettext()


class DpkgStatusCruft(Cruft):
    def __init__(self, n_items):
        self.n_items = n_items

    def get_prefix(self):
        return "dpkg-status"

    def get_prefix_description(self):
        return _("%i obsolete entries in the status file") % self.n_items

    def get_shortname(self):
        return _("Obsolete entries in dpkg status")

    def get_description(self):  # pragma: no cover
        return _("Obsolete dpkg status entries")

    def cleanup(self):
        logging.debug("calling dpkg --forget-old-unavail")
        res = subprocess.call("dpkg --forget-old-unavail".split())
        logging.debug("dpkg --forget-old-unavail returned {}".format(res))


class DpkgStatusPlugin(Plugin):
    def __init__(self, filename=None):
        self.status = "/var/lib/dpkg/status" if filename is None else filename
        self.condition = ["PostCleanup"]

    def get_cruft(self):
        n_cruft = 0
        with open(self.status) as fp:
            tagf = TagFile(fp)
            while tagf.step():
                statusline = tagf.section.get("Status")
                (want, flag, status) = statusline.split()
                if (
                    want == "purge"
                    and flag == "ok"
                    and status == "not-installed"
                ):
                    # Then...
                    n_cruft += 1
        logging.debug("DpkgStatusPlugin found {} cruft items".format(n_cruft))
        if n_cruft:
            return [DpkgStatusCruft(n_cruft)]
        return []



#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License as
#  published by the Free Software Foundation; either version 2 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
#  USA

import datetime
from gi.repository import GLib
import http.client
from mock import Mock
import logging
import sys
import unittest
import yaml

from UpdateManager.Core.LivePatchSocket import LivePatchSocket, LivePatchFix


status0 = {
    "architecture": "x86_64",
    "boot-time": datetime.datetime(2017, 6, 27, 11, 16),
    "client-version": "7.21",
    "cpu-model": "Intel(R) Core(TM) i7-6700HQ CPU @ 2.60GHz",
    "last-check": datetime.datetime(2017, 6, 28, 14, 23, 29, 683361),
    "machine-id": 123456789,
    "machine-token": 987654321,
    "uptime": "27h12m12s",
}

status1 = {
    "architecture": "x86_64",
    "boot-time": datetime.datetime(2017, 6, 27, 11, 16),
    "client-version": "7.21",
    "cpu-model": "Intel(R) Core(TM) i7-6700HQ CPU @ 2.60GHz",
    "last-check": datetime.datetime(2017, 6, 28, 14, 23, 29, 683361),
    "machine-id": 123456789,
    "machine-token": 987654321,
    "status": [
        {
            "kernel": "4.4.0-78.99-generic",
            "livepatch": {
                "checkState": "needs-check",
                "fixes": "",
                "patchState": "nothing-to-apply",
                "version": "24.2",
            },
            "running": True,
        }
    ],
    "uptime": "27h12m12s",
}

status2 = {
    "architecture": "x86_64",
    "boot-time": datetime.datetime(2017, 6, 27, 11, 16),
    "client-version": "7.21",
    "cpu-model": "Intel(R) Core(TM) i7-6700HQ CPU @ 2.60GHz",
    "last-check": datetime.datetime(2017, 6, 28, 14, 23, 29, 683361),
    "machine-id": 123456789,
    "machine-token": 987654321,
    "status": [
        {
            "kernel": "4.4.0-78.99-generic",
            "livepatch": {
                "checkState": "checked",
                "fixes": "* CVE-2016-0001\n"
                "* CVE-2016-0002\n"
                "* CVE-2017-0001 (unpatched)\n"
                "* CVE-2017-0001",
                "patchState": "applied",
                "version": "24.2",
            },
            "running": True,
        }
    ],
    "uptime": "27h12m12s",
}


class TestUtils(object):
    @staticmethod
    def __TimeoutCallback(user_data=None):
        user_data[0] = True
        return False

    @staticmethod
    def __ScheduleTimeout(timeout_reached, timeout_duration=10):
        return GLib.timeout_add(
            timeout_duration, TestUtils.__TimeoutCallback, timeout_reached
        )

    @staticmethod
    def WaitUntilMSec(
        instance,
        check_function,
        expected_result=True,
        max_wait=500,
        error_msg="",
    ):
        instance.assertIsNotNone(check_function)

        timeout_reached = [False]
        timeout_id = TestUtils.__ScheduleTimeout(timeout_reached, max_wait)

        result = None
        while not timeout_reached[0]:
            result = check_function()
            if result == expected_result:
                break
            GLib.MainContext.default().iteration(True)

        if result == expected_result:
            GLib.Source.remove(timeout_id)

        instance.assertEqual(expected_result, result, error_msg)


class MockResponse:
    def __init__(self, status, data):
        self.status = status
        self.data = data

    def read(self):
        return yaml.dump(self.data)


class TestLivePatchSocket(unittest.TestCase):
    def test_get_check_state(self):
        check_state = LivePatchSocket.get_check_state(status0)
        self.assertEqual(check_state, "check-failed")
        check_state = LivePatchSocket.get_check_state(status1)
        self.assertEqual(check_state, "needs-check")
        check_state = LivePatchSocket.get_check_state(status2)
        self.assertEqual(check_state, "checked")

    def test_get_patch_state(self):
        patch_state = LivePatchSocket.get_patch_state(status0)
        self.assertEqual(patch_state, "unknown")
        patch_state = LivePatchSocket.get_patch_state(status1)
        self.assertEqual(patch_state, "nothing-to-apply")
        patch_state = LivePatchSocket.get_patch_state(status2)
        self.assertEqual(patch_state, "applied")

    def test_get_fixes(self):
        fixes = LivePatchSocket.get_fixes(status0)
        self.assertEqual(fixes, [])
        fixes = LivePatchSocket.get_fixes(status1)
        self.assertEqual(fixes, [])
        fixes = LivePatchSocket.get_fixes(status2)
        self.assertEqual(
            fixes,
            [
                LivePatchFix("CVE-2016-0001"),
                LivePatchFix("CVE-2016-0002"),
                LivePatchFix("CVE-2017-0001 (unpatched)"),
                LivePatchFix("CVE-2017-0001"),
            ],
        )

    def test_livepatch_fix(self):
        fix = LivePatchFix("CVE-2016-0001")
        self.assertEqual(fix.name, "CVE-2016-0001")
        self.assertTrue(fix.patched)

        fix = LivePatchFix("CVE-2016-0001 (unpatched)")
        self.assertEqual(fix.name, "CVE-2016-0001")
        self.assertFalse(fix.patched)

    def test_callback_not_active(self):
        mock_http_conn = Mock(spec=http.client.HTTPConnection)
        attrs = {"getresponse.return_value": MockResponse(400, None)}
        mock_http_conn.configure_mock(**attrs)

        cb_called = [False]

        def on_done(active, check_state, patch_state, fixes):
            cb_called[0] = True
            self.assertFalse(active)

        lp = LivePatchSocket(mock_http_conn)
        lp.get_status(on_done)

        mock_http_conn.request.assert_called_with(
            "GET", "/status?verbose=True"
        )
        TestUtils.WaitUntilMSec(self, lambda: cb_called[0] is True)

    def test_callback_active(self):
        mock_http_conn = Mock(spec=http.client.HTTPConnection)
        attrs = {"getresponse.return_value": MockResponse(200, status2)}
        mock_http_conn.configure_mock(**attrs)

        cb_called = [False]

        def on_done(active, check_state, patch_state, fixes):
            cb_called[0] = True
            self.assertTrue(active)
            self.assertEqual(check_state, "checked")
            self.assertEqual(patch_state, "applied")
            self.assertEqual(
                fixes,
                [
                    LivePatchFix("CVE-2016-0001"),
                    LivePatchFix("CVE-2016-0002"),
                    LivePatchFix("CVE-2017-0001 (unpatched)"),
                    LivePatchFix("CVE-2017-0001"),
                ],
            )

        lp = LivePatchSocket(mock_http_conn)
        lp.get_status(on_done)

        mock_http_conn.request.assert_called_with(
            "GET", "/status?verbose=True"
        )
        TestUtils.WaitUntilMSec(self, lambda: cb_called[0] is True)


if __name__ == "__main__":
    if len(sys.argv) > 1 and sys.argv[1] == "-v":
        logging.basicConfig(level=logging.DEBUG)
    unittest.main()



import datetime
import gettext

gettext.install("update-manager")
from gettext import gettext as _


# the day on which the short support HWE stack goes EoL
HWE_EOL_DATE = datetime.date(2031, 5, 29)

# the day on which the next LTS first point release is available
# used to propose a release upgrade
NEXT_LTS_DOT1_DATE = datetime.date(2028, 7, 31)

# end of the month in which this LTS goes EoL
LTS_EOL_DATE = datetime.date(2031, 5, 29)


class Messages:
    UM_UPGRADE = (
        _(
            """
There is a graphics stack installed on this system. An upgrade to a
configuration supported for the full lifetime of the LTS will become
available on %(date)s and can be installed by running 'update-manager'
in the Dash.
    """
        )
        % {"date": NEXT_LTS_DOT1_DATE.isoformat()}
    )

    APT_UPGRADE = _(
        """
To upgrade to a supported (or longer-supported) configuration:

* Upgrade from Ubuntu 24.04 LTS to Ubuntu 26.04 LTS by running:
sudo do-release-upgrade %s

OR

* Switch to the current security-supported stack by running:
sudo apt-get install %s

and reboot your system."""
    )

    # this message is shown if there is no clear upgrade path via a
    # meta pkg that we recognize
    APT_SHOW_UNSUPPORTED = _(
        """
The following packages are no longer supported:
 %s

Please upgrade them to a supported HWE stack or remove them if you
no longer need them.
"""
    )

    HWE_SUPPORTED = _(
        "Your Hardware Enablement Stack (HWE) is "
        "supported until %(month)s %(year)s."
    ) % {"month": LTS_EOL_DATE.strftime("%B"), "year": LTS_EOL_DATE.year}

    HWE_SUPPORT_ENDS = (
        _(
            """
Your current Hardware Enablement Stack (HWE) is going out of support
on %s.  After this date security updates for critical parts (kernel
and graphics stack) of your system will no longer be available.

For more information, please see:
https://documentation.ubuntu.com/release-notes/26.04/
"""
        )
        % HWE_EOL_DATE.isoformat()
    )

    HWE_SUPPORT_HAS_ENDED = (
        _(
            """
WARNING: Security updates for your current Hardware Enablement
Stack ended on %s:
 * http://wiki.ubuntu.com/2204_HWE_EOL
"""
        )
        % HWE_EOL_DATE.isoformat()
    )



# Copyright (C) 2009-2012  Canonical, Ltd.
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, version 3 of the License.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

"""Install kdelibs5-dev if kdeblibs4-dev is installed."""

__metaclass__ = type
__all__ = ["Kdelibs4devToKdelibs5devPlugin"]


from janitor.plugincore.core.missing_package_cruft import MissingPackageCruft
from janitor.plugincore.i18n import setup_gettext
from janitor.plugincore.plugin import Plugin

_ = setup_gettext()


class Kdelibs4devToKdelibs5devPlugin(Plugin):
    """Plugin to install kdelibs5-dev if kdelibs4-dev is installed.

    See also LP: #279621.
    """

    def __init__(self):
        self.condition = ["from_hardyPostDistUpgradeCache"]

    def get_cruft(self):
        fromp = "kdelibs4-dev"
        top = "kdelibs5-dev"
        cache = self.app.apt_cache
        if (
            fromp in cache
            and cache[fromp].is_installed
            and top in cache
            and not cache[top].is_installed
        ):
            yield MissingPackageCruft(
                cache[top],
                _(
                    "When upgrading, if kdelibs4-dev is installed, "
                    "kdelibs5-dev needs to be installed. See "
                    "bugs.launchpad.net, bug #279621 for details."
                ),
            )



#!/usr/bin/env python3
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-

import glob
import os
import pathlib
import re
from distutils.core import setup
from subprocess import check_output

from DistUtilsExtra.command import build_extra, build_help, build_i18n

disabled = []


def make_pep440_compliant(version: str) -> str:
    """Convert the version into a PEP440 compliant version."""
    if ":" in version:
        # Strip epoch
        version = version.split(":", 1)[1]
    public_version_re = re.compile(
        r"^([0-9][0-9.]*(?:(?:a|b|rc|.post|.dev)[0-9]+)*)\+?"
    )
    _, public, local = public_version_re.split(version, maxsplit=1)
    if not local:
        return version
    sanitized_local = re.sub("[+~]+", ".", local).strip(".")
    pep440_version = f"{public}+{sanitized_local}"
    assert re.match(
        "^[a-zA-Z0-9.]+$", sanitized_local
    ), f"'{pep440_version}' not PEP440 compliant"
    return pep440_version


def plugins():
    return []
    return [
        os.path.join("janitor/plugincore/plugins", name)
        for name in os.listdir("janitor/plugincore/plugins")
        if name.endswith("_plugin.py") and name not in disabled
    ]


for line in check_output(
    "dpkg-parsechangelog --format rfc822".split(), universal_newlines=True
).splitlines():
    header, colon, value = line.lower().partition(":")
    if header == "version":
        VERSION = make_pep440_compliant(value.strip())
        break
else:
    raise RuntimeError("No version found in debian/changelog")


class CustomBuild(build_extra.build_extra):
    def run(self):
        version_py = pathlib.Path("UpdateManager/UpdateManagerVersion.py")
        version_py.write_text(f"VERSION = '{VERSION}'\n", encoding="utf-8")
        build_extra.build_extra.run(self)


setup(
    name="update-manager",
    version=VERSION,
    packages=[
        "UpdateManager",
        "UpdateManager.backend",
        "UpdateManager.Core",
        "HweSupportStatus",
        "janitor",
        "janitor.plugincore",
    ],
    scripts=["update-manager", "ubuntu-security-status", "hwe-support-status"],
    data_files=[
        ("share/update-manager/gtkbuilder", glob.glob("data/gtkbuilder/*.ui")),
        ("share/man/man8", glob.glob("data/*.8")),
        ("share/GConf/gsettings/", ["data/update-manager.convert"]),
    ],
    cmdclass={
        "build": CustomBuild,
        "build_i18n": build_i18n.build_i18n,
        "build_help": build_help.build_help,
    },
)



# Copyright (C) 2008-2012  Canonical, Ltd.
# -*- Mode: Python; indent-tabs-mode: nil; tab-width: 4; coding: utf-8 -*-
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation, version 3 of the License.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along with
# this program.  If not, see <http://www.gnu.org/licenses/>.

__metaclass__ = type
__all__ = ["DebPluginTests"]


import unittest

from janitor.plugincore.plugins.deb_plugin import DebPlugin
from janitor.plugincore.testing.helpers import Application


class DebPluginTests(unittest.TestCase):
    def setUp(self):
        self.plugin = DebPlugin()
        self.app = Application()
        self.plugin.set_application(self.app)

    def test_no_cruft(self):
        self.assertEqual(self.plugin.get_cruft(), [])

    def test_post_cleanup_calls_commit(self):
        self.plugin.post_cleanup()
        self.assertTrue(self.app.commit_called)

    def test_post_cleanup_calls_refresh(self):
        self.plugin.post_cleanup()
        self.assertTrue(self.app.refresh_called)
